/* ------------------------------------------------------------------------------

	FILENAME
		InitializeMessage.c

	DESCRIPTION
		This file contains the message procedure that will be invoked when the Printing Manager
		issues the Initialize message.

	COPYRIGHT
		Copyright Apple Computer, Inc. 1991-1994
		All rights reserved. 
	
	INTERFACE ROUTINES
		InitializeMessageProc

	MODIFICATION HISTORY
		12/03/91		Sam Weiss	Initial Implementation
		12/22/93		dmh			Modified to use recommended approach for
										initializing global data.


------------------------------------------------------------------------------- */

#include <Types.h>
#include <Quickdraw.h>
#include <Memory.h>
#include <Resources.h>
#include <Dialogs.h>
#include <TextEdit.h>
#include <OSUtils.h>
#include <Packages.h>
#include <ToolUtils.h>
#include <Menus.h>
#include <String.h>
#include <Strings.h>
#include <Printing.h>
#include <FixMath.h>

#include <graphics routines.h>

#include <Collections.h>
#include <Messages.h>

#include	<PrintingManager.h>
#include	<PrintingMessages.h>

#include	<Additions.h>


extern long A5Size (void);
extern void A5Init (void *);


// Globals...


gxShape gSerialShape;


//------------------------------------------------------------------------------------

/*
	Name:					InitGlobalData
	
	Description:		initialize gloable data.  You should always do this
							outside of the routine in which you call
							NewMessageGlobals.  Otherwise, your compiler may
							reference your data using an invalid A5 register.

	Parameters:			none
	
	Returns:				none
	
	Preconditions:		none
								
	Postconditions:	none
	
*/

void InitGlobalData (void) {
	gSerialShape = NULL;
}


//------------------------------------------------------------------------------------

/*
	Name:					InitializeMessageProc
	
	Description:		Initialize the universal driver.

	Parameters:			long:				unused context
	
	Returns:				OSErr
	
	Preconditions:		none
								
	Postconditions:	none
	
*/

OSErr InitializeMessageProc (void) {

	OSErr anErr;
	
	anErr = NewMessageGlobals(A5Size(), A5Init);
	
	if (!anErr) InitGlobalData();
	
	return anErr;
}
