/*------------------------------------------------------------------------------
FILENAME
	Generic LaserWriter.r

DESCRIPTION
	This module contains all of the #defines which for the generic
	LaserWriter driver.
	
   12/18/93	- dmh - Made non-IIg-specific for the b3 seed.
	9/13/93 - dmh - Updated for the b2 seed.
	4/26/93 - dmh - Updated for the b1 seed.

COPYRIGHT
	 Copyright Apple Computer, Inc. 1989-1994
	 All rights reserved. 
-------------------------------------------------------------------------------- */

#include "Types.r"
#include "SysTypes.r"
#include "PrintingResTypes.r"

// the defines

#define vShortVersionStr		"1.0b3"
#define	kDriverName				"Generic LaserWriter"
#define	kCreatorType			'GLWR'
#define	kFileType				'pdvr'


/* ------------------------------------------------------------------------ */
/* 'vers' RESOURCE REQUIRED BY ALL PRINTER DRIVERS							*/
/* ------------------------------------------------------------------------ */

/*	The first byte of this resource must be zero or one for
	this driver to be compatible with GX v1.0.  The
	remainder of the data is ignored						*/

resource 'vers' (gxPrintingDriverBaseID, sysheap, purgeable) {
	0x0,
	0x0,
	release,
	0x0,
	verUS,
	"",
	""
};


// Normal 'vers' resources so we can tell what version we've got.

resource 'vers' (1, sysheap, purgeable) {
	0x1,
	0x0,
	beta,
	0x3,
	verUS,
	vShortVersionStr,
	vShortVersionStr",  Apple Computer, Inc. 1988-1994"
};


resource 'vers' (2, sysheap, purgeable) {
	0x1,
	0x0,
	beta,
	0x3,
	verUS,
	vShortVersionStr,
	"Sample Printer Driver"
};


/*------------------------------------------- the bundle ------------------------------------*/


resource 'BNDL' (gxPrintingDriverBaseID + 1, sysheap, purgeable ) {
	kCreatorType,
	0,
	{ 
		'ICN#', {  0, gxPrintingDriverBaseID + 2; 1, gxPrintingDriverBaseID + 3; 2, gxPrintingDriverBaseID + 4; 3, gxPrintingDriverBaseID + 5},
		'FREF', {  0, gxPrintingDriverBaseID + 2; 1, gxPrintingDriverBaseID + 3; 2, gxPrintingDriverBaseID + 4; 3, gxPrintingDriverBaseID + 5; 0, gxPrintingDriverBaseID + 1 }
	}
};

type kCreatorType as 'STR ';

resource kCreatorType ( 0 ) { "Generic LaserWriter Driver " vShortVersionStr };

resource 'FREF' (gxPrintingDriverBaseID + 1, sysheap, purgeable ) { kFileType, 0, "" };

resource 'FREF' (gxPrintingDriverBaseID + 2, sysheap, purgeable ) { 'dpnn', 0, "" };
resource 'FREF' (gxPrintingDriverBaseID + 3, sysheap, purgeable ) { 'dpns', 1, "" };
resource 'FREF' (gxPrintingDriverBaseID + 4, sysheap, purgeable ) { 'dpcn', 2, "" };
resource 'FREF' (gxPrintingDriverBaseID + 5, sysheap, purgeable ) { 'dpcs', 3, "" };

resource 'STR ' (gxPrintingDriverBaseID + 2, sysheap, purgeable ) { vShortVersionStr };

resource 'pfil' ( 1, sysheap, purgeable ) {

	{
		"PostScript"
	};
};

/* ------------------------- resources for scanning the status ----------------------*/

resource gxPostscriptScanningType ( gxPostscriptScanningID, sysheap, purgeable )
{
	0,
	{
	SimpleScan {"busy"}, SimpleScan { "printer busy" }, SimpleOffset {}, NoAction {};
	SimpleScan {"waiting"}, SimpleScan {"preparing data"}, SimpleOffset {}, NoAction {};	
	SimpleScan {"job:"}, SimpleScan {"User"}, SimpleOffset {}, NoAction {};
	}
};

resource gxPostscriptScanningType ( gxPostscriptScanningID + 1, sysheap, purgeable )
{
	0,
	{
	SimpleScan {"%%["}, SimpleScan {""}, SimpleOffset {}, NoAction {};
	SimpleScan {"]%%"}, SimpleScan {""}, SimpleOffset {}, NoAction {};
	SimpleScan {"PrinterError"}, SimpleScan {"Printer"}, SimpleOffset {}, NoAction {};
	}
};

/*-------------------------- resource for telling printing who we are ------------------------*/

// what type of communication does this device use (this is the default, as the actual
// value can be found in the desktop printer).

resource 'comm' (0, sysheap, purgeable )
{
	PAP
		{
			8,									// the flow quantum
			"\0x0BLaserWriter\0x00\0x00",		// the name of the printer
			0,
			0,
			0,
			0
		};
};


// select which imaging system to use, the ID of the override resource, 
// and the number of overrides we use

resource gxImagingSystemSelectorType (gxImagingSystemSelectorID, sysheap, purgeable )
{
	gxPostscriptPrinterType	// we use the PostScript system
};


// this resource allows us to customize the old API's behavior

resource 'cust' (-8192, sysheap, purgeable )
{
	300, 300,					// our DPI
	laserWriter,				// we use the default up driver (LaserWriter)
	{4, 4},						// pattern stretch of 4 (nearest thing to 300 dpi/72 dpi)
	gxDefaultOptionsTranslation	// default translator settings
};

// this resource tells the quickdraw driver about the resolution of the device

resource gxReslType ( gxReslID, sysheap, purgeable )
{
	rangeType,
		25, 1500,
		25,	1500,
		{ 300, 300 };
};

// this resource customizes the behavior of the universal driver, allowing us
// to use standard mechanisms for buffering and IO

resource gxUniversalIOPrefsType ( gxUniversalIOPrefsID, sysheap, purgeable  )
{
	standardIO,
	4,			// four buffers (univ driver WILL run with less if it needs to)
	1024*2,		// we want them to each be large enough for one scan line of data (at least)
	10,			// number of requests blocks to allocate for HIO
	600,		// wait a whole two minutes to open and close connections
	1200,		// as well as for reading and writing
};

resource gxPostscriptPrefsType ( gxPostscriptPrefsID, sysheap, purgeable  )	
{
	1,											// language level
	gxRGBSpace,									// shape color space
	0,											// flags
	1500,										// path limit
	31,											// gsave limit
	500,										// operand stack limit
	type1StreamType + type3StreamType,			// the font type that this printer supports
	434204										// vm
};

/*----------------------------------------- message overrides --------------------------------*/

// overrides for universal messages and where to find them in the jump table

#define firstOffset		4
#define	segmentID		0

resource gxOverrideType ( gxDriverImagingOverrideID, sysheap, purgeable  )
	{
		{
			gxPostscriptDoPageSetUp, segmentID, firstOffset,
		};
	};

resource gxOverrideType ( gxDriverUniversalOverrideID, sysheap, purgeable  )
	{
		{
			gxOpenConnection,  	segmentID, firstOffset + 4,
			gxCloseConnection, 	segmentID, firstOffset + 8,
			gxInitialize,  		segmentID, firstOffset + 12,
			gxDumpBuffer, 	 	segmentID, firstOffset + 16,
		};
	};

/*-------------------------------------------- LaserWriter paper types --------------------*/

resource 'ptyp'	( gxPrintingDriverBaseID + 1, sysheap, purgeable  )	{
	
	"US Letter",
	
		/* page rectangle */
	0x00000000,				/* 0.0 */
	0x00000000,				/* 0.0 */
	0x0247AE18,				/* 8.10667 */
	0x0308A3DC,				/* 10.7867 */

		/* paper rectangle */
	0xFFF1D70C,				/* -0.196666 */
	0xFFF870A8,				/* -0.104999 */
	0x0255D70C,				/* 8.30333 */
	0x031070A8,				/* 10.895 */
	
	usLetterBase,
	kCreatorType,

	inch,

	newStylePaperType,
	isDefaultPaperType,
	{
		'post',
		gxPrintingTagID,
		itemLocked,
		itemPersistent,
		0,
		"/letter dup where { pop load exec }{ pop } ifelse\n"
	}
};

resource 'ptyp' ( gxPrintingDriverBaseID + 2, sysheap, purgeable  )	{
	
	"US Legal",
	
		/* page rectangle */
	0x00000000,				/* 0.0 */
	0x00000000,				/* 0.0 */
	0x0247AE18,				/* 8.10667 */
	0x03E0A3DC,				/* 13.7867 */

		/* paper rectangle */
	0xFFF1D70C,				/* -0.196666 */
	0xFFF870A8,				/* -0.104999 */
	0x0255D70C,				/* 8.30333 */
	0x03E870A8,				/* 13.895 */

	usLegalBase,
	kCreatorType,

	inch,
	
	newStylePaperType,
	notDefaultPaperType,
	{
		'post',
		gxPrintingTagID,
		itemLocked,
		itemPersistent,
		0,
		"/legal dup where { pop load exec }{ pop } ifelse\n"
	}
};


resource 'ptyp' ( gxPrintingDriverBaseID + 3, sysheap, purgeable  )	{

	"A4 Letter",
	
		/* page rectangle */
	0x00000000,				/* 0.0 */
	0x00000000,				/* 0.0 */
	0x023851EC,				/* 7.89333 */
	0x0337AE18,				/* 11.44 */

		/* paper rectangle */
	0xFFF3A3C0,				/* -0.18 */
	0xFFF73D74,				/* -0.121666 */
	0x02467AE4,				/* 8.09 */
	0x0340EB88,				/* 11.5683 */

	a4LetterBase,
	kCreatorType,

	inch,
	
	newStylePaperType,
	notDefaultPaperType,
	{
		'post',
		gxPrintingTagID,
		itemLocked,
		itemPersistent,
		0,
		"/a4 dup where { pop load exec }{ pop } ifelse\n"
	}
};

resource 'ptyp' ( gxPrintingDriverBaseID + 4, sysheap, purgeable  )	{

	"B5 Letter",
	
		/* page rectangle */
	0x00000000,				/* 0.0 */
	0x00000000,				/* 0.0 */
	0x01E3D70C,				/* 6.72 */
	0x02C28F60,				/* 9.81333 */

		/* paper rectangle */
	0xFFEB5C2A,				/* -0.286666 */
	0x000751EC,				/* 0.101667 */
	0x01DE51EC,				/* 6.64333 */
	0x02CBCCD0,				/* 9.94167 */

	b5LetterBase,
	kCreatorType,

	inch,
	
	newStylePaperType,
	notDefaultPaperType,
	{
		'post',
		gxPrintingTagID,
		itemLocked,
		itemPersistent,
		0,
		"/b5 dup where { pop load exec }{ pop } ifelse\n"
	}	
};

resource 'ptyp' ( gxPrintingDriverBaseID + 5, sysheap, purgeable  )	{

	"US Letter (small)",
	
		/* page rectangle */
	0x00000000,				/* 0.0 */
	0x00000000,				/* 0.0 */
	0x023851F0,				/* 7.89333 */
	0x02E5999C,				/* 10.3 */

		/* paper rectangle */
	0xFFEA28F6,				/* -0.303333 */
	0xFFE6B854,				/* -0.351111 */
	0x024E28F8,				/* 8.19667 */
	0x02FEB854,				/* 10.6489 */

	usLetterBase,
	kCreatorType,

	inch,
	
	newStylePaperType,
	notDefaultPaperType,
	{
		'post',
		gxPrintingTagID,
		itemLocked,
		itemPersistent,
		0,
		"/lettersmall dup where { pop load exec }{ pop } ifelse\n"
	}	
};

resource 'ptyp' ( gxPrintingDriverBaseID + 6, sysheap, purgeable  )	{

	"A4 Letter (small)",
	
		/* page rectangle */
	0x00000000,				/* 0.0 */
	0x00000000,				/* 0.0 */
	0x0228F5C4,				/* 7.68 */
	0x0317C294,				/* 10.9967 */

		/* paper rectangle */
	0xFFEB5C28,				/* -0.286667 */
	0xFFE7147C,				/* -0.346111 */
	0x023ECCD0,				/* 7.98333 */
	0x0330C290,				/* 11.3439 */

	a4LetterBase,
	kCreatorType,

	inch,
	
	newStylePaperType,
	notDefaultPaperType,
	{
		'post',
		gxPrintingTagID,
		itemLocked,
		itemPersistent,
		0,
		"/a4small dup where { pop load exec }{ pop } ifelse\n"
	}	
};

/*--------------------------------------- old style paper types -----------------------*/

resource 'ptyp'	( gxPrintingDriverBaseID + 7, sysheap, purgeable  )	{
	
	"US Letter",
	
		/* page rectangle */
	0x00000000,				/* 0.0 */
	0x00000000,				/* 0.0 */
	0x0247AE18,				/* 8.10667 */
	0x0308A3DC,				/* 10.7867 */

		/* paper rectangle */
	0xFFF1D70C,				/* -0.196666 */
	0xFFF870A8,				/* -0.104999 */
	0x0255D70C,				/* 8.30333 */
	0x031070A8,				/* 10.895 */
	
	usLetterBase,
	kCreatorType,

	inch,

	oldStylePaperType,
	isDefaultPaperType,
	{
		'post',
		gxPrintingTagID,
		itemLocked,
		itemPersistent,
		0,
		"/letter dup where { pop load exec }{ pop } ifelse\n"
	}	
};

resource 'ptyp' ( gxPrintingDriverBaseID + 8, sysheap, purgeable  )	{
	
	"US Legal",
	
		/* page rectangle */
	0x00000000,				/* 0.0 */
	0x00000000,				/* 0.0 */
	0x0247AE18,				/* 8.10667 */
	0x03E0A3DC,				/* 13.7867 */

		/* paper rectangle */
	0xFFF1D70C,				/* -0.196666 */
	0xFFF870A8,				/* -0.104999 */
	0x0255D70C,				/* 8.30333 */
	0x03E870A8,				/* 13.895 */

	usLegalBase,
	kCreatorType,

	inch,
	
	oldStylePaperType,
	notDefaultPaperType,
	{
		'post',
		gxPrintingTagID,
		itemLocked,
		itemPersistent,
		0,
		"/legal dup where { pop load exec }{ pop } ifelse\n"
	}	
};


resource 'ptyp' ( gxPrintingDriverBaseID + 9, sysheap, purgeable  )	{

	"A4 Letter",
	
		/* page rectangle */
	0x00000000,				/* 0.0 */
	0x00000000,				/* 0.0 */
	0x023851EC,				/* 7.89333 */
	0x0337AE18,				/* 11.44 */

		/* paper rectangle */
	0xFFF3A3C0,				/* -0.18 */
	0xFFF73D74,				/* -0.121666 */
	0x02467AE4,				/* 8.09 */
	0x0340EB88,				/* 11.5683 */

	a4LetterBase,
	kCreatorType,

	inch,
	
	oldStylePaperType,
	notDefaultPaperType,
	{
		'post',
		gxPrintingTagID,
		itemLocked,
		itemPersistent,
		0,
		"/a4 dup where { pop load exec }{ pop } ifelse\n"
	}	
};

resource 'ptyp' ( gxPrintingDriverBaseID + 10, sysheap, purgeable  )	{

	"B5 Letter",
	
		/* page rectangle */
	0x00000000,				/* 0.0 */
	0x00000000,				/* 0.0 */
	0x01E3D70C,				/* 6.72 */
	0x02C28F60,				/* 9.81333 */

		/* paper rectangle */
	0xFFEB5C2A,				/* -0.286666 */
	0x000751EC,				/* 0.101667 */
	0x01DE51EC,				/* 6.64333 */
	0x02CBCCD0,				/* 9.94167 */

	b5LetterBase,
	kCreatorType,

	inch,
	
	oldStylePaperType,
	notDefaultPaperType,
	{
		'post',
		gxPrintingTagID,
		itemLocked,
		itemPersistent,
		0,
		"/b5 dup where { pop load exec }{ pop } ifelse\n"
	}	
};

resource 'ptyp' ( gxPrintingDriverBaseID + 11, sysheap, purgeable  )	{

	"US Letter (small)",
	
		/* page rectangle */
	0x00000000,				/* 0.0 */
	0x00000000,				/* 0.0 */
	0x023851F0,				/* 7.89333 */
	0x02E5999C,				/* 10.3 */

		/* paper rectangle */
	0xFFEA28F6,				/* -0.303333 */
	0xFFE6B854,				/* -0.351111 */
	0x024E28F8,				/* 8.19667 */
	0x02FEB854,				/* 10.6489 */

	usLetterBase,
	kCreatorType,

	inch,
	
	oldStylePaperType,
	notDefaultPaperType,
	{
		'post',
		gxPrintingTagID,
		itemLocked,
		itemPersistent,
		0,
		"/lettersmall dup where { pop load exec }{ pop } ifelse\n"
	}	
};

resource 'ptyp' ( gxPrintingDriverBaseID + 12, sysheap, purgeable  )	{

	"A4 Letter (small)",
	
		/* page rectangle */
	0x00000000,				/* 0.0 */
	0x00000000,				/* 0.0 */
	0x0228F5C4,				/* 7.68 */
	0x0317C294,				/* 10.9967 */

		/* paper rectangle */
	0xFFEB5C28,				/* -0.286667 */
	0xFFE7147C,				/* -0.346111 */
	0x023ECCD0,				/* 7.98333 */
	0x0330C290,				/* 11.3439 */

	a4LetterBase,
	kCreatorType,

	inch,
	
	oldStylePaperType,
	notDefaultPaperType,
	{
		'post',
		gxPrintingTagID,
		itemLocked,
		itemPersistent,
		0,
		"/a4small dup where { pop load exec }{ pop } ifelse\n"
	}	
};


/*-------------------------- resources for printer fonts ---------------------------------*/

resource 'pfnt' ( 128, "AvantGarde-Book", sysheap, purgeable )		{
	ROMFont,
	AdobeCharacterSet {};
};
resource 'pfnt' ( 129, "AvantGarde-BookOblique", sysheap, purgeable )		{
	ROMFont,
	AdobeCharacterSet {};
};
resource 'pfnt' ( 130, "AvantGarde-Demi", sysheap, purgeable )		{
	ROMFont,
	AdobeCharacterSet {};
};
resource 'pfnt' ( 131, "AvantGarde-DemiOblique", sysheap, purgeable )		{
	ROMFont,
	AdobeCharacterSet {};
};
resource 'pfnt' ( 132, "Bookman-Demi", sysheap, purgeable )		{
	ROMFont,
	AdobeCharacterSet {};
};
resource 'pfnt' ( 133, "Bookman-DemiItalic", sysheap, purgeable )		{
	ROMFont,
	AdobeCharacterSet {};
};
resource 'pfnt' ( 134, "Bookman-Light", sysheap, purgeable )		{
	ROMFont,
	AdobeCharacterSet {};
};
resource 'pfnt' ( 135, "Bookman-LightItalic", sysheap, purgeable )		{
	ROMFont,
	AdobeCharacterSet {};
};
resource 'pfnt' ( 136, "Courier", sysheap, purgeable )		{
	ROMFont,
	AdobeCharacterSet {};
};
resource 'pfnt' ( 137, "Courier-Bold", sysheap, purgeable )		{
	ROMFont,
	AdobeCharacterSet {};
};
resource 'pfnt' ( 138, "Courier-BoldOblique", sysheap, purgeable )		{
	ROMFont,
	AdobeCharacterSet {};
};
resource 'pfnt' ( 139, "Courier-Oblique", sysheap, purgeable )		{
	ROMFont,
	AdobeCharacterSet {};
};
resource 'pfnt' ( 140, "Helvetica", sysheap, purgeable )		{
	ROMFont,
	AdobeCharacterSet {};
};
resource 'pfnt' ( 141, "Helvetica-Bold", sysheap, purgeable )		{
	ROMFont,
	AdobeCharacterSet {};
};
resource 'pfnt' ( 142, "Helvetica-BoldOblique", sysheap, purgeable )		{
	ROMFont,
	AdobeCharacterSet {};
};
resource 'pfnt' ( 143, "Helvetica-Narrow", sysheap, purgeable )		{
	ROMFont,
	AdobeCharacterSet {};
};
resource 'pfnt' ( 144, "Helvetica-Narrow-Bold", sysheap, purgeable )		{
	ROMFont,
	AdobeCharacterSet {};
};
resource 'pfnt' ( 145, "Helvetica-Narrow-BoldOblique", sysheap, purgeable )		{
	ROMFont,
	AdobeCharacterSet {};
};
resource 'pfnt' ( 146, "Helvetica-Narrow-Oblique", sysheap, purgeable )		{
	ROMFont,
	AdobeCharacterSet {};
};
resource 'pfnt' ( 147, "Helvetica-Oblique", sysheap, purgeable )		{
	ROMFont,
	AdobeCharacterSet {};
};
resource 'pfnt' ( 148, "NewCenturySchlbk-Bold", sysheap, purgeable )		{
	ROMFont,
	AdobeCharacterSet {};
};
resource 'pfnt' ( 149, "NewCenturySchlbk-BoldItalic", sysheap, purgeable )		{
	ROMFont,
	AdobeCharacterSet {};
};
resource 'pfnt' ( 150, "NewCenturySchlbk-Italic", sysheap, purgeable )		{
	ROMFont,
	AdobeCharacterSet {};
};
resource 'pfnt' ( 151, "NewCenturySchlbk-Roman", sysheap, purgeable )		{
	ROMFont,
	AdobeCharacterSet {};
};
resource 'pfnt' ( 153, "Palatino-Bold", sysheap, purgeable )		{
	ROMFont,
	AdobeCharacterSet {};
};
resource 'pfnt' ( 154, "Palatino-BoldItalic", sysheap, purgeable )		{
	ROMFont,
	AdobeCharacterSet {};
};
resource 'pfnt' ( 155, "Palatino-Italic", sysheap, purgeable )		{
	ROMFont,
	AdobeCharacterSet {};
};
resource 'pfnt' ( 156, "Palatino-Roman", sysheap, purgeable )		{
	ROMFont,
	AdobeCharacterSet {};
};
resource 'pfnt' ( 157, "Symbol", sysheap, purgeable )		{
	ROMFont,
	AppleCharacterSet {
		191,
		$"9FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
	};	
};
resource 'pfnt' ( 158, "Times-Bold", sysheap, purgeable )		{
	ROMFont,
	AdobeCharacterSet {};
};
resource 'pfnt' ( 159, "Times-BoldItalic", sysheap, purgeable )		{
	ROMFont,
	AdobeCharacterSet {};
};
resource 'pfnt' ( 160, "Times-Italic", sysheap, purgeable )		{
	ROMFont,
	AdobeCharacterSet {};
};
resource 'pfnt' ( 161, "Times-Roman", sysheap, purgeable )		{
	ROMFont,
	AdobeCharacterSet {};
};
resource 'pfnt' ( 162, "ZapfChancery-MediumItalic", sysheap, purgeable )		{
	ROMFont,
	AdobeCharacterSet {};
};

/* -------------------------------- the icons ---------------------------------------- */

/* --- the driver --- */
/* --- the DTP, non-current, non-shared --- */

data 'ICN#' (gxPrintingDriverBaseID + 2, sysheap, purgeable ) {
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
	$"01FF FF00 0100 0100 0100 0100 010E E100"            /* ................ */
	$"0511 1100 0511 1100 0517 D100 0517 D100"            /* .............. */
	$"071E F100 0120 0900 0130 1900 012F E900"            /* ..... ..0.../.. */
	$"0F10 11E0 11FF FF10 2400 0048 42AA AA84"            /* ........$..HB */
	$"8000 0002 8000 0002 AAAA AAAA 8000 0002"            /* ......... */
	$"8000 0002 9C00 0002 8000 0002 7FFF FFFC"            /* ............. */
	$"1000 0010 2000 0008 2000 0008 1FFF FFF0"            /* .... ... ....... */
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
	$"01FF FF00 01FF FF00 01FF FF00 01FF FF00"            /* ................ */
	$"05FF FF00 05FF FF00 05FF FF00 05FF FF00"            /* ................ */
	$"07FF FF00 01FF FF00 01FF FF00 01FF FF00"            /* ................ */
	$"0FFF FFE0 1FFF FFF0 3FFF FFF8 7FFF FFFC"            /* ........?....... */
	$"FFFF FFFE FFFF FFFE FFFF FFFE FFFF FFFE"            /* ................ */
	$"FFFF FFFE FFFF FFFE FFFF FFFE 7FFF FFFC"            /* ................ */
	$"1FFF FFF0 3FFF FFF8 3FFF FFF8 1FFF FFF0"            /* ....?...?....... */
};

data 'icl4' (gxPrintingDriverBaseID + 2, sysheap, purgeable ) {
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
	$"0000 000F FFFF FFFF FFFF FFFF 0000 0000"            /* ................ */
	$"0000 000F 0C0C 0C0C 0C0C 0C0F 0000 0000"            /* ................ */
	$"0000 000F C0C0 C0C0 C0C0 C0CF 0000 0000"            /* ........ */
	$"0000 000F 0C0C FFFC FFFC 0C0F 0000 0000"            /* ................ */
	$"0000 0F0F C0CF 000F 000F C0CF 0000 0000"            /* ............ */
	$"0000 0F0F 0C0F 000F 000F 0C0F 0000 0000"            /* ................ */
	$"0000 0F0F C0CF 0FFF FF0F C0CF 0000 0000"            /* ............ */
	$"0000 0F0F 0C0F 0FFF FF0F 0C0F 0000 0000"            /* ................ */
	$"0000 0FFF C008 FFF8 FFF8 C0CF 0000 0000"            /* ............. */
	$"0000 000F 0C88 8888 8888 8C0F 0000 0000"            /* .......... */
	$"0000 000F C088 8888 8888 80CF 0000 0000"            /* ........ */
	$"0000 000F 0CEE 8888 888E EC0F 0000 0000"            /* ............ */
	$"0000 FFFF C008 EEEE EEE8 00CF FFF0 0000"            /* .............. */
	$"000F 001F FFFF FFFF FFFF FFFF 001F 0000"            /* ................ */
	$"00F0 1B00 1000 1000 1000 1000 1B00 F000"            /* ................ */
	$"0F10 00B0 B0B0 B0B0 B0B0 B0B0 B010 0F00"            /* ...... */
	$"F000 1000 1000 1000 1000 1000 1000 10F0"            /* ................ */
	$"F010 0010 0010 0010 0010 0010 0010 00F0"            /* ................ */
	$"F0B0 B0B0 B0B0 B0B0 B0B0 B0B0 B0B0 B0F0"            /* .. */
	$"F010 0010 0010 0010 0010 0010 0010 00F0"            /* ................ */
	$"F000 1000 1000 1000 1000 1000 1000 10F0"            /* ................ */
	$"F016 6610 0010 0010 0010 0010 0010 00F0"            /* ..f............. */
	$"F000 1000 1000 1000 1000 1000 1000 10F0"            /* ................ */
	$"0FFF FFFF FFFF FFFF FFFF FFFF FFFF FF00"            /* ................ */
	$"000F EEEE EEEE EEEE EEEE EEEE EEEF 0000"            /* ................ */
	$"00FD DDDD DDDD DDDD DDDD DDDD DDDD F000"            /* ................ */
	$"00FC CCCC CCCC CCCC CCCC CCCC CCCC F000"            /* .... */
	$"000F FFFF FFFF FFFF FFFF FFFF FFFF 0000"            /* ................ */
};

data 'icl8' (gxPrintingDriverBaseID + 2, sysheap, purgeable ) {
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 00FF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 00FF 00F7 00F7 00F7 00F7"            /* ................ */
	$"00F7 00F7 00F7 00FF 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 00FF F700 F700 F700 F700"            /* ................ */
	$"F700 F700 F700 F7FF 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 00FF 00F7 00F7 FFFF FFF7"            /* ................ */
	$"FFFF FFF7 00F7 00FF 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 00FF 00FF F700 F7FF 0000 00FF"            /* ................ */
	$"0000 00FF F700 F7FF 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 00FF 00FF 00F7 00FF 0000 00FF"            /* ................ */
	$"0000 00FF 00F7 00FF 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 00FF 00FF F700 F7FF 00FF FFFF"            /* ................ */
	$"FFFF 00FF F700 F7FF 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 00FF 00FF 00F7 00FF 00FF FFFF"            /* ................ */
	$"FFFF 00FF 00F7 00FF 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 00FF FFFF F700 00E4 FFFF FFE4"            /* ................ */
	$"FFFF FFE4 F700 F7FF 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 00FF 00F7 E4E4 E4E4 E4E4"            /* ................ */
	$"E4E4 E4E4 E4F7 00FF 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 00FF F700 E4E4 E4E4 E4E4"            /* ................ */
	$"E4E4 E4E4 E400 F7FF 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 00FF 00F7 A5A5 E4E4 E4E4"            /* .............. */
	$"E4E4 E4A5 A5F7 00FF 0000 0000 0000 0000"            /* .............. */
	$"0000 0000 FFFF FFFF F700 00E4 A5A5 A5A5"            /* ............ */
	$"A5A5 A5E4 0000 F7FF FFFF FF00 0000 0000"            /* ............. */
	$"0000 00FF 0101 01FF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF 0101 01FF 0000 0000"            /* ................ */
	$"0000 FF01 0133 0101 0101 0101 0101 0101"            /* .....3.......... */
	$"0101 0101 0101 0101 0133 0101 FF00 0000"            /* .........3...... */
	$"00FF 0101 0101 3301 3301 3301 3301 3301"            /* ......3.3.3.3.3. */
	$"3301 3301 3301 3301 3301 0101 01FF 0000"            /* 3.3.3.3.3....... */
	$"FF01 0101 0101 0101 0101 0101 0101 0101"            /* ................ */
	$"0101 0101 0101 0101 0101 0101 0101 FF00"            /* ................ */
	$"FF01 0101 0101 0101 0101 0101 0101 0101"            /* ................ */
	$"0101 0101 0101 0101 0101 0101 0101 FF00"            /* ................ */
	$"FF01 3301 3301 3301 3301 3301 3301 3301"            /* ..3.3.3.3.3.3.3. */
	$"3301 3301 3301 3301 3301 3301 3301 FF00"            /* 3.3.3.3.3.3.3... */
	$"FF01 0101 0101 0101 0101 0101 0101 0101"            /* ................ */
	$"0101 0101 0101 0101 0101 0101 0101 FF00"            /* ................ */
	$"FF01 0101 0101 0101 0101 0101 0101 0101"            /* ................ */
	$"0101 0101 0101 0101 0101 0101 0101 FF00"            /* ................ */
	$"FF01 01EC ECEC 0101 0101 0101 0101 0101"            /* ................ */
	$"0101 0101 0101 0101 0101 0101 0101 FF00"            /* ................ */
	$"FF01 0101 0101 0101 0101 0101 0101 0101"            /* ................ */
	$"0101 0101 0101 0101 0101 0101 0101 FF00"            /* ................ */
	$"00FF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF 0000"            /* ................ */
	$"0000 00FF FBFB FBFB FBFB FBFB FBFB FBFB"            /* ................ */
	$"FBFB FBFB FBFB FBFB FBFB FBFF 0000 0000"            /* ................ */
	$"0000 FFF9 F9F9 F9F9 F9F9 F9F9 F9F9 F9F9"            /* ................ */
	$"F9F9 F9F9 F9F9 F9F9 F9F9 F9F9 FF00 0000"            /* ................ */
	$"0000 FFF7 F7F7 F7F7 F7F7 F7F7 F7F7 F7F7"            /* ................ */
	$"F7F7 F7F7 F7F7 F7F7 F7F7 F7F7 FF00 0000"            /* ................ */
	$"0000 00FF FFFF FFFF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFFF FFFF 0000 0000"            /* ................ */
};

/* small icons not currently used */

data 'ics#' (gxPrintingDriverBaseID + 2, sysheap, purgeable ) {
	$"0000 0FF8 0808 2808 2948 3808 0A28 19CC"            /* ......(.)H8..(. */
	$"2FFA 4001 5555 4001 5801 7FFF 3FFE 0000"            /* /.@.UU@.X...?... */
	$"0000 0FF8 0FF8 2FF8 2FF8 3FF8 0FF8 1FFC"            /* .....././.?..... */
	$"3FFE 7FFF 7FFF 7FFF 7FFF 7FFF 3FFE 0000"            /* ?...........?... */
};

data 'ics4' (gxPrintingDriverBaseID + 2, sysheap, purgeable ) {
	$"0000 0000 0000 0000 0000 FFFF FFFF F000"            /* ................ */
	$"0000 F000 0000 F000 00F0 F0CE CEC0 F000"            /* ............. */
	$"00F0 F0E0 E0E0 F000 00FF F0EF EFE0 F000"            /* ................ */
	$"0000 F098 8890 F000 000F F089 9980 FF00"            /* .......... */
	$"00F0 FFFF FFFF F0F0 0F10 0010 0010 001F"            /* ................ */
	$"0F0B 1B0B 1B0B 1B0F 0F10 0010 0010 001F"            /* ................ */
	$"0F0F F000 1000 100F 0FFF FFFF FFFF FFFF"            /* ................ */
	$"00FF FFFF FFFF FFF0 0000 0000 0000 0000"            /* ................ */
};

data 'ics8' (gxPrintingDriverBaseID + 2, sysheap, purgeable ) {
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 FFFF FFFF FFFF FFFF FF00 0000"            /* ................ */
	$"0000 0000 FF00 0000 0000 0000 FF00 0000"            /* ................ */
	$"0000 FF00 FF00 F8FB F8FB F800 FF00 0000"            /* ................ */
	$"0000 FF00 FF00 FB00 FB00 FB00 FF00 0000"            /* ................ */
	$"0000 FFFF FF00 FBFF FBFF FB00 FF00 0000"            /* ................ */
	$"0000 0000 FF00 E7E1 E1E1 E700 FF00 0000"            /* ................ */
	$"0000 00FF FF00 E1E7 E7E7 E100 FFFF 0000"            /* ................ */
	$"0000 FF01 FFFF FFFF FFFF FFFF FF01 FF00"            /* ................ */
	$"00FF 0101 0101 0101 0101 0101 0101 01FF"            /* ................ */
	$"00FF 0133 0133 0133 0133 0133 0133 01FF"            /* ...3.3.3.3.3.3.. */
	$"00FF 0101 0101 0101 0101 0101 0101 01FF"            /* ................ */
	$"00FF 01FF FF01 0101 0101 0101 0101 01FF"            /* ................ */
	$"00FF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"            /* ................ */
	$"0000 FFFF FFFF FFFF FFFF FFFF FFFF FF00"            /* ................ */
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
};

/* --- the DTP, non-current, shared --- */

data 'ICN#' (gxPrintingDriverBaseID + 3, sysheap, purgeable ) {
	$"01FF FF00 0100 0100 0100 0100 010E E100"            /* ................ */
	$"0511 1100 0511 1100 0517 D100 0517 D100"            /* .............. */
	$"071F F100 013F F900 013F F900 013F F900"            /* .....?...?...?.. */
	$"0F1F F1E0 11FF FF10 2000 0008 4000 0004"            /* ........ ...@... */
	$"8000 0002 8000 0002 8000 0002 8000 0002"            /* ............ */
	$"8000 0002 9C00 0002 8000 0002 7FFF FFFC"            /* ............. */
	$"1FFF FFF0 2000 0008 1FFF FFF0 0000 02A0"            /* .... .......... */
	$"BFFF FC9D 0000 0140 BFFF FE3D 0000 0000"            /* .....@..=.... */
	$"01FF FF00 01FF FF00 01FF FF00 01FF FF00"            /* ................ */
	$"05FF FF00 05FF FF00 05FF FF00 05FF FF00"            /* ................ */
	$"07FF FF00 01FF FF00 01FF FF00 01FF FF00"            /* ................ */
	$"0FFF FFE0 1FFF FFF0 3FFF FFF8 7FFF FFFC"            /* ........?....... */
	$"FFFF FFFE FFFF FFFE FFFF FFFE FFFF FFFE"            /* ................ */
	$"FFFF FFFE FFFF FFFE FFFF FFFE 7FFF FFFC"            /* ................ */
	$"1FFF FFF0 3FFF FFF8 1FFF FFF0 0000 03E0"            /* ....?........... */
	$"FFFF FFFF FFFF FF7F FFFF FE3F 0000 0000"            /* ...........?.... */
};

data 'icl4' (gxPrintingDriverBaseID + 3, sysheap, purgeable ) {
	$"0000 000F FFFF FFFF FFFF FFFF 0000 0000"            /* ................ */
	$"0000 000F 0C0C 0C0C 0C0C 0C0F 0000 0000"            /* ................ */
	$"0000 000F C0C0 C0C0 C0C0 C0CF 0000 0000"            /* ........ */
	$"0000 000F 0C0C FFFC FFFC 0C0F 0000 0000"            /* ................ */
	$"0000 0F0F C0CF 000F 000F C0CF 0000 0000"            /* ............ */
	$"0000 0F0F 0C0F 000F 000F 0C0F 0000 0000"            /* ................ */
	$"0000 0F0F C0CF 0FFF FF0F C0CF 0000 0000"            /* ............ */
	$"0000 0F0F 0C0F 0FFF FF0F 0C0F 0000 0000"            /* ................ */
	$"0000 0FFF C008 FFF8 FFF8 C0CF 0000 0000"            /* ............. */
	$"0000 000F 0C88 8888 8888 8C0F 0000 0000"            /* .......... */
	$"0000 000F C088 8888 8888 80CF 0000 0000"            /* ........ */
	$"0000 000F 0CEE 8888 888E EC0F 0000 0000"            /* ............ */
	$"0000 FFFF C008 EEEE EEE8 00CF FFF0 0000"            /* .............. */
	$"000F 010F FFFF FFFF FFFF FFFF 010F 0000"            /* ................ */
	$"00F0 1D10 1010 1010 1010 1010 1D10 F000"            /* ................ */
	$"0F01 01D1 D1D1 D1D1 D1D1 D1D1 0101 0F00"            /* ....... */
	$"F010 1010 1010 1010 1010 1010 1010 10F0"            /* ................ */
	$"F101 0101 0101 0101 0101 0101 0101 01F0"            /* ................ */
	$"F0D0 D0D0 D0D0 D0D0 D0D0 D0D0 D0D0 D0F0"            /* .. */
	$"F101 0101 0101 0101 0101 0101 0101 01F0"            /* ................ */
	$"F010 1010 1010 1010 1010 1010 1010 10F0"            /* ................ */
	$"F106 6601 0101 0101 0101 0101 0101 01F0"            /* ..f............. */
	$"F010 1010 1010 1010 1010 1010 1010 10F0"            /* ................ */
	$"0FFF FFFF FFFF FFFF FFFF FFFF FFFF FF00"            /* ................ */
	$"000F EEEE EEEE EEEE EEEE EEEE EEEF 0000"            /* ................ */
	$"00FD DDDD DDDD DDDD DDDD DDDD DDDD F000"            /* ................ */
	$"000F FFFF FFFF FFFF FFFF FFFF FFFF 0000"            /* ................ */
	$"0000 0000 0000 0000 0000 00FC FCE0 0000"            /* ................ */
	$"CCCC CCCC CCDD DDDD EEEE EECC FCCE EEEE"            /* ......... */
	$"0000 00CC CCCC CCCC CCCC CCCF 0FCC CCCC"            /* .... */
	$"CCCC CDDD DDDE EEEE EEEE EFF0 00FE EEEE"            /* ............. */
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
};

data 'icl8' (gxPrintingDriverBaseID + 3, sysheap, purgeable ) {
	$"0000 0000 0000 00FF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 00FF 00F7 00F7 00F7 00F7"            /* ................ */
	$"00F7 00F7 00F7 00FF 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 00FF F700 F700 F700 F700"            /* ................ */
	$"F700 F700 F700 F7FF 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 00FF 00F7 00F7 FFFF FFF7"            /* ................ */
	$"FFFF FFF7 00F7 00FF 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 00FF 00FF F700 F7FF 0000 00FF"            /* ................ */
	$"0000 00FF F700 F7FF 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 00FF 00FF 00F7 00FF 0000 00FF"            /* ................ */
	$"0000 00FF 00F7 00FF 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 00FF 00FF F700 F7FF 00FF FFFF"            /* ................ */
	$"FFFF 00FF F700 F7FF 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 00FF 00FF 00F7 00FF 00FF FFFF"            /* ................ */
	$"FFFF 00FF 00F7 00FF 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 00FF FFFF F700 00E4 FFFF FFE4"            /* ................ */
	$"FFFF FFE4 F700 F7FF 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 00FF 00F7 E4E4 E4E4 E4E4"            /* ................ */
	$"E4E4 E4E4 E4F7 00FF 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 00FF F700 E4E4 E4E4 E4E4"            /* ................ */
	$"E4E4 E4E4 E400 F7FF 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 00FF 00F7 A5A5 E4E4 E4E4"            /* .............. */
	$"E4E4 E4A5 A5F7 00FF 0000 0000 0000 0000"            /* .............. */
	$"0000 0000 FFFF FFFF F700 00E4 A5A5 A5A5"            /* ............ */
	$"A5A5 A5E4 0000 F7FF FFFF FF00 0000 0000"            /* ............. */
	$"0000 00FF 0101 01FF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF 0101 01FF 0000 0000"            /* ................ */
	$"0000 FF01 0133 0101 0101 0101 0101 0101"            /* .....3.......... */
	$"0101 0101 0101 0101 0133 0101 FF00 0000"            /* .........3...... */
	$"00FF 0101 0101 3301 3301 3301 3301 3301"            /* ......3.3.3.3.3. */
	$"3301 3301 3301 3301 3301 0101 01FF 0000"            /* 3.3.3.3.3....... */
	$"FF01 0101 0101 0101 0101 0101 0101 0101"            /* ................ */
	$"0101 0101 0101 0101 0101 0101 0101 FF00"            /* ................ */
	$"FF01 0101 0101 0101 0101 0101 0101 0101"            /* ................ */
	$"0101 0101 0101 0101 0101 0101 0101 FF00"            /* ................ */
	$"FF01 3301 3301 3301 3301 3301 3301 3301"            /* ..3.3.3.3.3.3.3. */
	$"3301 3301 3301 3301 3301 3301 3301 FF00"            /* 3.3.3.3.3.3.3... */
	$"FF01 0101 0101 0101 0101 0101 0101 0101"            /* ................ */
	$"0101 0101 0101 0101 0101 0101 0101 FF00"            /* ................ */
	$"FF01 0101 0101 0101 0101 0101 0101 0101"            /* ................ */
	$"0101 0101 0101 0101 0101 0101 0101 FF00"            /* ................ */
	$"FF01 01EC ECEC 0101 0101 0101 0101 0101"            /* ................ */
	$"0101 0101 0101 0101 0101 0101 0101 FF00"            /* ................ */
	$"FF01 0101 0101 0101 0101 0101 0101 0101"            /* ................ */
	$"0101 0101 0101 0101 0101 0101 0101 FF00"            /* ................ */
	$"00FF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF 0000"            /* ................ */
	$"0000 00FF FBFB FBFB FBFB FBFB FBFB FBFB"            /* ................ */
	$"FBFB FBFB FBFB FBFB FBFB FBFF 0000 0000"            /* ................ */
	$"0000 FFF9 F9F9 F9F9 F9F9 F9F9 F9F9 F9F9"            /* ................ */
	$"F9F9 F9F9 F9F9 F9F9 F9F9 F9F9 FF00 0000"            /* ................ */
	$"0000 00FF FFFF FFFF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFFF FFFF 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 FF2B FE2B FD00 0000 0000"            /* .......+.+...... */
	$"F6F6 F6F6 F7F7 F7F8 F8F8 F9F9 F9FA FAFA"            /* ................ */
	$"FBFB FBFC FCFC 2B2B FE2B 2BFD FDFC FCFC"            /* ......++.++..... */
	$"F5F5 F5F5 F5F5 F6F6 F6F6 F6F6 F62B 2B2B"            /* .............+++ */
	$"2B2B 2B2B 2B2B 2BFE 00FE 2B2B 2B2B 2B2B"            /* +++++++...++++++ */
	$"F7F7 F8F8 F8F9 F9F9 FAFA FAFB FBFB FCFC"            /* ................ */
	$"FCFD FDFD FEFE FE00 0000 FEFE FDFD FCFC"            /* ................ */
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
};

/* --- the DTP, current, non-shared --- */

data 'ICN#' (gxPrintingDriverBaseID + 4, sysheap, purgeable ) {
	$"0000 0000 0000 0000 03FF FF80 07FF FFC0"            /* .............. */
	$"07FF FFC0 0700 01C0 0700 01C0 470E E1C0"            /* .........G.. */
	$"E711 11C0 E711 11C0 E717 D1C0 FF17 D1C0"            /* .......... */
	$"FF1F F1C0 7F3F F9C0 073F F9E0 073F F9F0"            /* ....?..?...?.. */
	$"0F1F F1F8 1DFF FF3C 3C00 004E 72AA AA87"            /* .......<<..Nr */
	$"E000 0007 E000 0007 EAAA AAAF E000 0007"            /* ............. */
	$"E000 0007 FC00 0007 E000 0007 FFFF FFFF"            /* ................ */
	$"FFFF FFFE FFFF FFFE FFFF FFFE 7FFF FFFC"            /* ................ */
	$"0000 0000 0000 0000 03FF FF80 07FF FFC0"            /* .............. */
	$"07FF FFC0 07FF FFC0 07FF FFC0 47FF FFC0"            /* .........G.. */
	$"E7FF FFC0 E7FF FFC0 E7FF FFC0 FFFF FFC0"            /* ............ */
	$"FFFF FFC0 7FFF FFC0 07FF FFE0 07FF FFF0"            /* .............. */
	$"0FFF FFF8 1FFF FFFC 3FFF FFFE 7FFF FFFF"            /* ........?....... */
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFE FFFF FFFE FFFF FFFE 7FFF FFFC"            /* ................ */
};

data 'icl4' (gxPrintingDriverBaseID + 4, sysheap, purgeable ) {
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
	$"0000 00FF FFFF FFFF FFFF FFFF F000 0000"            /* ................ */
	$"0000 0FFF FFFF FFFF FFFF FFFF FF00 0000"            /* ................ */
	$"0000 0FFF FFFF FFFF FFFF FFFF FF00 0000"            /* ................ */
	$"0000 0FFF 0C0C 0C0C 0C0C 0C0F FF00 0000"            /* ................ */
	$"0000 0FFF C0C0 C0C0 C0C0 C0CF FF00 0000"            /* ........ */
	$"0F00 0FFF 0C0C FFFC FFFC 0C0F FF00 0000"            /* ................ */
	$"FFF0 0FFF C0CF 000F 000F C0CF FF00 0000"            /* ............ */
	$"FFF0 0FFF 0C0F 000F 000F 0C0F FF00 0000"            /* ................ */
	$"FFF0 0FFF C0CF 0FFF FF0F C0CF FF00 0000"            /* ............ */
	$"FFFF FFFF 0C0F 0FFF FF0F 0C0F FF00 0000"            /* ................ */
	$"FFFF FFFF C008 FFF8 FFF8 C0CF FF00 0000"            /* ............. */
	$"0FFF FFFF 0C88 8888 8888 8C0F FF00 0000"            /* .......... */
	$"0000 0FFF C088 8888 8888 80CF FFF0 0000"            /* ........ */
	$"0000 0FFF 0CEE 8888 888E EC0F FFFF 0000"            /* ............ */
	$"0000 FFFF C008 EEEE EEE8 00CF FFFF F000"            /* .............. */
	$"000F FF0F FFFF FFFF FFFF FFFF 01DF FF00"            /* ................ */
	$"00FF FD10 1010 1010 1010 1010 1D10 FFF0"            /* ................ */
	$"0FFF 01D1 D1D1 D1D1 D1D1 D1D1 D101 0FFF"            /* ...... */
	$"FFF0 1010 1010 1010 1010 1010 1010 1FFF"            /* ................ */
	$"FFF1 0101 0101 0101 0101 0101 0101 0FFF"            /* ................ */
	$"FFF0 D0D0 D0D0 D0D0 D0D0 D0D0 D0D0 DFFF"            /* .... */
	$"FFF1 0101 0101 0101 0101 0101 0101 0FFF"            /* ................ */
	$"FFF0 1010 1010 1010 1010 1010 1010 1FFF"            /* ................ */
	$"FFF6 6601 0101 0101 0101 0101 0101 0FFF"            /* ..f............. */
	$"FFF0 1010 1010 1010 1010 1010 1010 1FFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF EEEE EEEE EEEE EEEE EEEE EEEF FFF0"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFF0"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFF0"            /* ................ */
	$"0FFF FFFF FFFF FFFF FFFF FFFF FFFF FF00"            /* ................ */
};

data 'icl8' (gxPrintingDriverBaseID + 4, sysheap, purgeable ) {
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 FFFF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF FF00 0000 0000 0000"            /* ................ */
	$"0000 0000 00FF FFFF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFFF 0000 0000 0000"            /* ................ */
	$"0000 0000 00FF FFFF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFFF 0000 0000 0000"            /* ................ */
	$"0000 0000 00FF FFFF 00F7 00F7 00F7 00F7"            /* ................ */
	$"00F7 00F7 00F7 00FF FFFF 0000 0000 0000"            /* ................ */
	$"0000 0000 00FF FFFF F700 F700 F700 F700"            /* ................ */
	$"F700 F700 F700 F7FF FFFF 0000 0000 0000"            /* ................ */
	$"00FF 0000 00FF FFFF 00F7 00F7 FFFF FFF7"            /* ................ */
	$"FFFF FFF7 00F7 00FF FFFF 0000 0000 0000"            /* ................ */
	$"FFFF FF00 00FF FFFF F700 F7FF 0000 00FF"            /* ................ */
	$"0000 00FF F700 F7FF FFFF 0000 0000 0000"            /* ................ */
	$"FFFF FF00 00FF FFFF 00F7 00FF 0000 00FF"            /* ................ */
	$"0000 00FF 00F7 00FF FFFF 0000 0000 0000"            /* ................ */
	$"FFFF FF00 00FF FFFF F700 F7FF 00FF FFFF"            /* ................ */
	$"FFFF 00FF F700 F7FF FFFF 0000 0000 0000"            /* ................ */
	$"FFFF FFFF FFFF FFFF 00F7 00FF 00FF FFFF"            /* ................ */
	$"FFFF 00FF 00F7 00FF FFFF 0000 0000 0000"            /* ................ */
	$"FFFF FFFF FFFF FFFF F700 00E4 FFFF FFE4"            /* ................ */
	$"FFFF FFE4 F700 F7FF FFFF 0000 0000 0000"            /* ................ */
	$"00FF FFFF FFFF FFFF 00F7 E4E4 E4E4 E4E4"            /* ................ */
	$"E4E4 E4E4 E4F7 00FF FFFF 0000 0000 0000"            /* ................ */
	$"0000 0000 00FF FFFF F700 E4E4 E4E4 E4E4"            /* ................ */
	$"E4E4 E4E4 E400 F7FF FFFF FF00 0000 0000"            /* ................ */
	$"0000 0000 00FF FFFF 00F7 A5A5 E4E4 E4E4"            /* .............. */
	$"E4E4 E4A5 A5F7 00FF FFFF FFFF 0000 0000"            /* .............. */
	$"0000 0000 FFFF FFFF F700 00E4 A5A5 A5A5"            /* ............ */
	$"A5A5 A5E4 0000 F7FF FFFF FFFF FF00 0000"            /* ............. */
	$"0000 00FF FFFF 01FF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF 0101 33FF FFFF 0000"            /* ..........3..... */
	$"0000 FFFF FF33 0101 0101 0101 0101 0101"            /* .....3.......... */
	$"0101 0101 0101 0101 0133 0101 FFFF FF00"            /* .........3...... */
	$"00FF FFFF 0101 3301 3301 3301 3301 3301"            /* ......3.3.3.3.3. */
	$"3301 3301 3301 3301 3301 0101 01FF FFFF"            /* 3.3.3.3.3....... */
	$"FFFF FF01 0101 0101 0101 0101 0101 0101"            /* ................ */
	$"0101 0101 0101 0101 0101 0101 01FF FFFF"            /* ................ */
	$"FFFF FF01 0101 0101 0101 0101 0101 0101"            /* ................ */
	$"0101 0101 0101 0101 0101 0101 01FF FFFF"            /* ................ */
	$"FFFF FF01 3301 3301 3301 3301 3301 3301"            /* ....3.3.3.3.3.3. */
	$"3301 3301 3301 3301 3301 3301 33FF FFFF"            /* 3.3.3.3.3.3.3... */
	$"FFFF FF01 0101 0101 0101 0101 0101 0101"            /* ................ */
	$"0101 0101 0101 0101 0101 0101 01FF FFFF"            /* ................ */
	$"FFFF FF01 0101 0101 0101 0101 0101 0101"            /* ................ */
	$"0101 0101 0101 0101 0101 0101 01FF FFFF"            /* ................ */
	$"FFFF FFEC ECEC 0101 0101 0101 0101 0101"            /* ................ */
	$"0101 0101 0101 0101 0101 0101 01FF FFFF"            /* ................ */
	$"FFFF FF01 0101 0101 0101 0101 0101 0101"            /* ................ */
	$"0101 0101 0101 0101 0101 0101 01FF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FBFB FBFB FBFB FBFB FBFB FBFB"            /* ................ */
	$"FBFB FBFB FBFB FBFB FBFB FBFF FFFF FF00"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FF00"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FF00"            /* ................ */
	$"00FF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF 0000"            /* ................ */
};

/* --- the DTP, current, shared --- */

data 'ICN#' (gxPrintingDriverBaseID + 5, sysheap, purgeable ) {
	$"07FF FFC0 07FF FFC0 07FF FFC0 4700 01C0"            /* .........G.. */
	$"E70E E1C0 E711 11C0 FF17 D1C0 FF17 D1C0"            /* .......... */
	$"7F1F F1C0 0F3F F9C0 1F3F F9E0 3F3F F9F0"            /* ....?..?..??.. */
	$"7F1F F1F8 F1FF FF3C E800 003E E555 554F"            /* .......<...>.UUO */
	$"E000 0007 E000 0007 EAAA AAAF E000 0007"            /* ............. */
	$"E000 0007 FC00 0007 E000 000F FFFF FFFF"            /* ................ */
	$"FFFF FFFF 7FFF FFFE 3FFF FFFC 0000 02A0"            /* ........?...... */
	$"BFFF FC9D 0000 0140 BFFF FE3D 0000 0000"            /* .....@..=.... */
	$"07FF FFC0 07FF FFC0 07FF FFC0 47FF FFC0"            /* .........G.. */
	$"E7FF FFC0 E7FF FFC0 FFFF FFC0 FFFF FFC0"            /* ............ */
	$"7FFF FFC0 0FFF FFC0 1FFF FFE0 3FFF FFF0"            /* ..........?... */
	$"7FFF FFF8 FFFF FFFC FFFF FFFE FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF 7FFF FFFE 3FFF FFFC 0000 03E0"            /* ........?....... */
	$"FFFF FFFF FFFF FF7F FFFF FE3F 0000 0000"            /* ...........?.... */
};

data 'icl4' (gxPrintingDriverBaseID + 5, sysheap, purgeable ) {
	$"0000 0FFF FFFF FFFF FFFF FFFF FF00 0000"            /* ................ */
	$"0000 0FFF FFFF FFFF FFFF FFFF FF00 0000"            /* ................ */
	$"0000 0FFF FFFF FFFF FFFF FFFF FF00 0000"            /* ................ */
	$"0F00 0FFF 0C00 0000 0000 0C0F FF00 0000"            /* ................ */
	$"FFF0 0FFF C0CC FFFC FFFC C0CF FF00 0000"            /* ............ */
	$"FFF0 0FFF 0C0F 000F 000F 0C0F FF00 0000"            /* ................ */
	$"FFFF FFFF C0CF 0FFF FF0F C0CF FF00 0000"            /* ............ */
	$"FFFF FFFF 0C0F 0FFF FF0F 0C0F FF00 0000"            /* ................ */
	$"0FFF FFFF C008 FFF8 FFF8 C0CF FF00 0000"            /* ............. */
	$"0000 FFFF 0C88 8888 8888 8C0F FF00 0000"            /* .......... */
	$"000F FFFF C088 8888 8888 80CF FFF0 0000"            /* ........ */
	$"00FF FFFF 0CEE 8888 888E EC0F FFFF 0000"            /* ............ */
	$"0FFF FFFF C008 EEEE EEE8 00CF FFFF F000"            /* .............. */
	$"FFFF 010F FFFF FFFF FFFF FFFF 01FF FF00"            /* ................ */
	$"FFF0 1D10 1010 1010 1010 1010 1D1F FFF0"            /* ................ */
	$"FFF1 01D1 D1D1 D1D1 D1D1 D1D1 D101 FFFF"            /* ...... */
	$"FFF0 1010 1010 1010 1010 1010 1010 1FFF"            /* ................ */
	$"FFF1 0101 0101 0101 0101 0101 0101 0FFF"            /* ................ */
	$"FFF0 D0D0 D0D0 D0D0 D0D0 D0D0 D0D0 DFFF"            /* .... */
	$"FFF1 0101 0101 0101 0101 0101 0101 0FFF"            /* ................ */
	$"FFF0 1010 1010 1010 1010 1010 1010 1FFF"            /* ................ */
	$"FFF6 6601 0101 0101 0101 0101 0101 0FFF"            /* ..f............. */
	$"FFF0 1010 1010 1010 1010 1010 1010 FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"            /* ................ */
	$"0FFF FFFF FFFF FFFF FFFF FFFF FFFF FFF0"            /* ................ */
	$"00FF FFFF FFFF FFFF FFFF FFFF FFFF FF00"            /* ................ */
	$"0000 0000 0000 0000 0000 00FC FCA0 0000"            /* ............... */
	$"CCCC CCCC CCDD DDDD EEEE EECC FCCA AEEE"            /* .......ʮ. */
	$"0000 00CC CCCC CCCC CCCC CCCF 0FCC CCCC"            /* .... */
	$"CCCC CDDD DDDE EEEE EAAA FFF0 00FF AAEE"            /* ........... */
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
};

data 'icl8' (gxPrintingDriverBaseID + 5, sysheap, purgeable ) {
	$"0000 0000 00FF FFFF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFFF 0000 0000 0000"            /* ................ */
	$"0000 0000 00FF FFFF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFFF 0000 0000 0000"            /* ................ */
	$"0000 0000 00FF FFFF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFFF 0000 0000 0000"            /* ................ */
	$"00FF 0000 00FF FFFF 00F7 0000 0000 0000"            /* ................ */
	$"0000 0000 00F7 00FF FFFF 0000 0000 0000"            /* ................ */
	$"FFFF FF00 00FF FFFF F700 F7F7 FFFF FFF7"            /* ................ */
	$"FFFF FFF7 F700 F7FF FFFF 0000 0000 0000"            /* ................ */
	$"FFFF FF00 00FF FFFF 00F7 00FF 0000 00FF"            /* ................ */
	$"0000 00FF 00F7 00FF FFFF 0000 0000 0000"            /* ................ */
	$"FFFF FFFF FFFF FFFF F700 F7FF 00FF FFFF"            /* ................ */
	$"FFFF 00FF F700 F7FF FFFF 0000 0000 0000"            /* ................ */
	$"FFFF FFFF FFFF FFFF 00F7 00FF 00FF FFFF"            /* ................ */
	$"FFFF 00FF 00F7 00FF FFFF 0000 0000 0000"            /* ................ */
	$"00FF FFFF FFFF FFFF F700 00E4 FFFF FFE4"            /* ................ */
	$"FFFF FFE4 F700 F7FF FFFF 0000 0000 0000"            /* ................ */
	$"0000 0000 FFFF FFFF 00F7 E4E4 E4E4 E4E4"            /* ................ */
	$"E4E4 E4E4 E4F7 00FF FFFF 0000 0000 0000"            /* ................ */
	$"0000 00FF FFFF FFFF F700 E4E4 E4E4 E4E4"            /* ................ */
	$"E4E4 E4E4 E400 F7FF FFFF FF00 0000 0000"            /* ................ */
	$"0000 FFFF FFFF FFFF 00F7 A5A5 E4E4 E4E4"            /* .............. */
	$"E4E4 E4A5 A5F7 00FF FFFF FFFF 0000 0000"            /* .............. */
	$"00FF FFFF FFFF FFFF F700 00E4 A5A5 A5A5"            /* ............ */
	$"A5A5 A5E4 0000 F7FF FFFF FFFF FF00 0000"            /* ............. */
	$"FFFF FFFF 0101 01FF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF 0101 FFFF FFFF 0000"            /* ................ */
	$"FFFF FF01 0133 0101 0101 0101 0101 0101"            /* .....3.......... */
	$"0101 0101 0101 0101 0133 01FF FFFF FF00"            /* .........3...... */
	$"FFFF FF01 0101 3301 3301 3301 3301 3301"            /* ......3.3.3.3.3. */
	$"3301 3301 3301 3301 3301 0101 FFFF FFFF"            /* 3.3.3.3.3....... */
	$"FFFF FF01 0101 0101 0101 0101 0101 0101"            /* ................ */
	$"0101 0101 0101 0101 0101 0101 01FF FFFF"            /* ................ */
	$"FFFF FF01 0101 0101 0101 0101 0101 0101"            /* ................ */
	$"0101 0101 0101 0101 0101 0101 01FF FFFF"            /* ................ */
	$"FFFF FF01 3301 3301 3301 3301 3301 3301"            /* ....3.3.3.3.3.3. */
	$"3301 3301 3301 3301 3301 3301 33FF FFFF"            /* 3.3.3.3.3.3.3... */
	$"FFFF FF01 0101 0101 0101 0101 0101 0101"            /* ................ */
	$"0101 0101 0101 0101 0101 0101 01FF FFFF"            /* ................ */
	$"FFFF FF01 0101 0101 0101 0101 0101 0101"            /* ................ */
	$"0101 0101 0101 0101 0101 0101 01FF FFFF"            /* ................ */
	$"FFFF FFEC ECEC 0101 0101 0101 0101 0101"            /* ................ */
	$"0101 0101 0101 0101 0101 0101 01FF FFFF"            /* ................ */
	$"FFFF FF01 0101 0101 0101 0101 0101 0101"            /* ................ */
	$"0101 0101 0101 0101 0101 0101 FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"            /* ................ */
	$"00FF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FF00"            /* ................ */
	$"0000 FFFF FFFF FFFF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF 0000"            /* ................ */
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 FF2B FE2B FD00 0000 0000"            /* .......+.+...... */
	$"F6F6 F6F6 F7F7 F7F8 F8F8 F9F9 F9FA FAFA"            /* ................ */
	$"FBFB FBFC FCFC 2B2B FE2B 2BFD FDFC FCFC"            /* ......++.++..... */
	$"F5F5 F5F5 F5F5 F6F6 F6F6 F6F6 F62B 2B2B"            /* .............+++ */
	$"2B2B 2B2B 2B2B 2BFE 00FE 2B2B 2B2B 2B2B"            /* +++++++...++++++ */
	$"F7F7 F8F8 F8F9 F9F9 FAFA FAFB FBFB FCFC"            /* ................ */
	$"FCFD FDFD FEFE FE00 0000 FEFE FDFD FCFC"            /* ................ */
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
};

