/******************************************************************************\
*
* Apple Macintosh Developer Technical Support
*
* Main program file
*
* Program: DavesFabSamples
* File:    DavesFabSamples.c
*
* by:      Forrest Tanaka
*
* Copyright  1988-1990 Apple Computer, Inc.
* All rights reserved.
*
\******************************************************************************/


/******************************************************************************\
* Header Files
\******************************************************************************/

#ifndef THINK_C
#include <Desk.h>
#include <DiskInit.h>
#include <Errors.h>
#include <Fonts.h>
#include <Menus.h>
#include <Resources.h>
#include <ToolUtils.h>
#endif

#include <AppleEvents.h>
#include <GestaltEqu.h>
#include <Packages.h>
#include <Traps.h>

#include <graphics routines.h>

#include "EmergMem.h"
#include "Exceptions.h"
#include "DavesFabSamples.h"
#include "MenuHandler.h"
#include "WindowPositioner.h"


/******************************************************************************\
* Constants
\******************************************************************************/

/* Disk initialization */
#define kSysAlertLeft 80 /* Left coord of DIBadMount alert in screen coords */
#define kSysAlertTop  80 /* Top coord of DIBadMount alert in screen coords */

/* EqualString convenience constants */
#define kCaseSens true /* Pass to EqualString for case-sensitive check */
#define kDiacSens true /* Pass to EqualString for diacritical-sens. check */

/* Constants for checking on command-. */
#define kMaskModifiers 0xFE00     /* Mask for modifiers w/o command key */
#define kMaskASCII1    0x00FF0000 /* get the key out of the ASCII1 byte */
#define kMaskASCII2    0x000000FF /* get the key out of the ASCII2 byte */

/* Miscellaneous */
#define kMinWindSize 128 /* Minimum size of a window */
#define kMaxSleepTime 60 /* # ticks willing to wait between minor switches */


/******************************************************************************\
* Macros
\******************************************************************************/

/* Return status of bth bit of m */
#define btst(m,b) ((m) & (1L << (b)))


/******************************************************************************\
* Global Variables
\******************************************************************************/

Boolean gQuitting;       /* True if user requested that this app quit */
Boolean gWereInFront;    /* True if this application is frontmost */
Boolean gFixMenus;       /* True if menus need fixing */
Boolean gHasAppleEvents; /* True if Apple Events implemented */
Boolean gHasCoolSF;      /* True if 7.0 Standard File routines available */


/******************************************************************************\
* Private Function Prototypes
\******************************************************************************/

int main(void);

void StartUp(void);

void gxShutDown(void);

void EventLoop(void);

long CalcSleepTime(void);

void DoMouseDown(
	EventRecord *anEvent);

void DoKeyDown(
	EventRecord *anEvent);

void DoDiskEvt(
	EventRecord *anEvent);

pascal OSErr HandleAEquit(
	AppleEvent *quitAppleEvent,
	AppleEvent *reply,
	long       handlerRefCon);

OSErr DoneRequiredParams(
	AppleEvent *anAppleEvent);

void DoWindowClose(
	EventRecord *anEvent,
	WindowPtr   eventWindow);

short MaxToolboxTrap(void);

TrapType GetTrapType(
	short theTrap);


/******************************************************************************\
* Public: main - Entry gxPoint to this application
*
* After the gxHeap is initialized by allocating several master pointer blocks and
* expanding the applications gxHeap to its maximum size, StartUp is called to
* complete initialization.  Then the main event loop is entered, and thats
* where we stay until the user chooses Quit.
\******************************************************************************/

int main()
{
	/* Prepare the gxHeap */
	MaxApplZone();
	MoreMasters();
	MoreMasters();
	MoreMasters();
	MoreMasters();
	MoreMasters();
	MoreMasters();
	MoreMasters();
	MoreMasters();

	/* gxInitialize the application */
	StartUp();

	/* Execute the main event loop */
	EventLoop();

	/* Shut down the application */
	gxShutDown();

	/* Return the ANSI way */
	return 0;
}


/******************************************************************************\
* StartUp - Do whatever has to be done to gxInitialize the application
*
* This routine is called after the gxHeap is initialized to gxInitialize the
* application.  This involves initializing the toolbox, emergency memory,
* loading up the menus, validating the current environment, and initializing
* global variables.  If any errors occur while doing this, StartUp displays
* an alert telling the user what the error was and then ExitToShell is called.
* This is an unusual way to react to errors, and I only do it here because its
* so early in execution that there really isnt much else that can be done.
*
* See EmergMem.h in this application for details about emergency memory.
\******************************************************************************/

static void StartUp()
{
	short        result;  /* Result of alert; ignored */
	long         aeAttrs; /* AppleEvent attributes */
	long         sfAttrs; /* Standard File attributes */
	OSErr        error;
	ExceptionRec exception;
	long         msgType;
	long         msgCode;

	if (ExceptionEntry( /*<*/&exception, /*<*/&msgType, /*<*/&msgCode ))
	{
		result = ShowAlert( kStopAlert, rOKAlertID, msgType, msgCode );
		ExitToShell();
	}

	/* gxInitialize Sarano */
	GXEnterGraphics();

	/* gxInitialize the toolbox */
	InitGraf( &thePort );
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs( nil );

	/* gxInitialize emergency memory */
	InitEmergMem();
	if (FailLowMemory( 0 ))
		Exception( &exception, rMemErrMessages, kMemErrAppOpenMsg );

	/* gxInitialize the menus */
	error = StartMenus();
	if (error == memFullErr || FailLowMemory( 0 ))
		Exception( &exception, rMemErrMessages, kMemErrAppOpenMsg );
	else if (error == resNotFound)
		Exception( &exception, rResErrMessages, kResErrAppDamageMsg );
	else if (error == dsSysErr)
		Exception( &exception, rMiscErrMessages, kMiscErrUnknownMsg );

	/* Load and lock the disk-initialization package */
	DILoad();

	/* Check for the fancier capabilities */
	error = Gestalt( gestaltAppleEventsAttr, /*<*/&aeAttrs );
	if (error != noErr)
		gHasAppleEvents = false;
	else
		gHasAppleEvents = btst( aeAttrs, gestaltAppleEventsPresent );
	error = Gestalt( gestaltStandardFileAttr, /*<*/&sfAttrs );
	if (error != noErr)
		gHasCoolSF = false;
	else
		gHasCoolSF = btst( sfAttrs, gestaltStandardFile58 );

	/* Install the AppleEvent handler */
	if (gHasAppleEvents)
	{
		error = AEInstallEventHandler( kCoreEventClass, kAEQuitApplication,
				(EventHandlerProcPtr)HandleAEquit, 0, false );
		if (error == memFullErr || FailLowMemory( 0 ))
			Exception( &exception, rMemErrMessages, kMemErrAppOpenMsg );
		else if (error != noErr)
			Exception( &exception, rMiscErrMessages, kMiscErrUnknownMsg );
	}
}


/******************************************************************************\
* gxShutDown - Do whatever has to be done to shut down the application
*
* This routine is called when the application is about to shut down.  It calls
* GXExitGraphics to shut down Sarano graphics.
\******************************************************************************/

static void gxShutDown()
{
	GXExitGraphics();
}


/******************************************************************************\
* EventLoop - Main event loop for this application
*
* This is the main event loop of this application.  During every iteration of
* the event loop, the menus are kept up-to-date.  Also, NoEmergMem is called to
* detect whether the emergency memory was used.  If it was, then RecoverEmergMem
* is called in an attept to get it back.  If it cant, then some commands could
* be disabled until the memory can be recovered.
\******************************************************************************/

static void EventLoop()
{
	EventRecord anEvent;    /* An incoming event */
	Boolean     gotEvent;   /* True if a non-null event was received */
	WindowPtr   lastWindow; /* Pointer to front window during last iteration */
	WindowPtr   currWindow; /* Pointer to the current front window */

	gWereInFront = true;
	gQuitting = false;
	gFixMenus = true;
	lastWindow = nil;
	InitCursor();

	/* We loop until gQuitting is true */
	while (!gQuitting)
	{
		/* Try to reallocate emergency memory if its been used */
		if (NoEmergMem())
			RecoverEmergMem();

		/* Fix the menus to reflect current conditions */
		currWindow = FrontWindow();
		if (currWindow != lastWindow || gFixMenus)
		{
			FixMenus();
			lastWindow = currWindow;
			gFixMenus = false;
		}

		/* Its time to get and examine an event */
		gotEvent = WaitNextEvent( everyEvent, /*<*/&anEvent,
				CalcSleepTime(), nil );
		if (gotEvent)
			switch (anEvent.what)
			{
				case mouseDown:
					DoMouseDown( &anEvent );
					break;
				case keyDown:
				case autoKey:
					DoKeyDown( &anEvent );
					break;
				case updateEvt:
					DoUpdateEvt( &anEvent );
					break;
				case diskEvt:
					DoDiskEvt( &anEvent );
					break;
				case kHighLevelEvent:
					(void)AEProcessAppleEvent( &anEvent );
					break;
			}
	}
}


/******************************************************************************\
* CalcSleepTime - Calculate the number of ticks for WaitNextEvent to sleep
*
* WaitNextEvent takes a parameter that specifies the number of ticks that this
* application allows WaitNextEvent to service other open applications.  This is
* just a guideline; the actual amount of time that WaitNextEvent takes could be
* shorter or longer than our specification, depending on how heavy the system
* load is and on how long other applications take.
*\*****************************************************************************/

static long CalcSleepTime()
{
	long sleepTime; /* Number of ticks to sleep */

	sleepTime = kMaxSleepTime;

	return sleepTime;
}


/******************************************************************************\
* DoMouseDown - Mouse-down event dispatcher
*
* When a mouseDown event is received in the main event loop, this routine is
* called to determine which area on the screens the mouseDown was, and to
* dispatch to the appropriate routine to handle mouseDown events in that area.
* The mouseDown event is passed in the anEvent parameter.
*
* See MenuHandler.h for routines that handle mouse-down events in the menu bar.
\******************************************************************************/

static void DoMouseDown(
	EventRecord *anEvent) /* Contains mouse-down event */
{
	short     clickArea;   /* Area of the screen that was clicked */
	WindowPtr eventWindow; /* Pointer the clicked window, if any */

	/* Find clicked area of screen or window */
	clickArea = FindWindow( anEvent->where, /*<*/&eventWindow );

	/* Jump to mouseDown-handling routine appropriate for screen area */
	switch (clickArea)
	{
		case inMenuBar:
			DoMenuChoice( MenuSelect( anEvent->where ) );
			break;
		case inGoAway:
			DoWindowClose( anEvent, eventWindow );
			break;
		default:
			break;
	}
}


/******************************************************************************\
* DoKeyDown - Key-down event dispatcher
*
* When a keyDown or autoKey event is received in the main event loop, this
* routine is called to determine whether key is a command-key equivalent for a
* menu item or not.  If the command key isnt down, then the key stroke is
* ignored.  Otherwise, MenuKey is called to get the menu ID and item number
* of the menu item that corresponds to the command key, if any.  Then
* DoMenuChoice is called to dispatch to the appropriate routine for the chosen
* menu item.  The keyDown or autoKey event is passed in anEvent.
\******************************************************************************/

static void DoKeyDown(
	EventRecord *anEvent) /* Contains the key-down event */
{
	char theKey; /* ASCII code of key that was pressed */

	/* Get the ASCII code of the pressed key */
	theKey = anEvent->message & charCodeMask;

	/* If anEvent was keyDown and command key was down, its menu command */
	if (anEvent->what == keyDown && (anEvent->modifiers & cmdKey))
		DoMenuChoice( MenuKey( theKey ) );
}


/******************************************************************************\
* Private: DoWindowClose - Handle a click in the close box of a window
*
* This routine should be called when the user clicks in the close box of the
* window specified by eventWind.  The mouse is tracked until the user releases
* the mouse button.  If the user released the mouse button while the mouse was
* in the close box, then DisposeWindow is called to close the window.  anEvent
* contains the mouse-down event that was determined to be a mouse click in the
* window.
\******************************************************************************/

static void DoWindowClose(
	EventRecord *anEvent,    /* Mouse-down event in the close box */
	WindowPtr   eventWindow) /* Pointer to the window that was clicked */
{
	if (TrackGoAway( eventWindow, anEvent->where ))
		DisposeWindow( eventWindow );
}


/******************************************************************************\
* DoDiskEvt - Handle a disk-insert event
*
* This routine is called whenever this application receives an event indicating
* that a disk was inserted.  If the disk cant be mounted, the message field of
* the event reflects the error, and we call DIBadMount to allow the user to
* gxFormat the disk.
\******************************************************************************/

static void DoDiskEvt(
	EventRecord *anEvent) /* Disk-insert event */
{
	Point cornerPoint; /* Top-left corner of DIBadMount alert */

	if (hiWord( anEvent->message ) != noErr)
	{
		SetPt( /*<*/&cornerPoint, kSysAlertLeft, kSysAlertTop );
		(void)DIBadMount( cornerPoint, anEvent->message );
	}
}


/******************************************************************************\
* Private: HandleAEquit - Handler for 'quit' AppleEvent
*
* This is the AppleEvent handler for the 'quit' AppleEvent as passed in the
* quitAppleEvent parameter by the AppleEvent Manager.  The DoQuit routine is
* called which causes this application to quit at the start of the next
* iteration of the main event loop.
*
* Though the quit AppleEvent doesnt contain any parameters, the standard thing
* to do in reaction to any AppleEvent is to check to see if there are any
* required parameters in the AppleEvent that this routine doesnt recognise.
* DoneRequiredParms checks for this condition and returns an error if there are
* in fact required parameters in the AppleEvent or if some other error occurs
* during the check.
\******************************************************************************/

static pascal OSErr HandleAEquit(
	AppleEvent *quitAppleEvent, /* Contains the quit AppleEvent */
	AppleEvent *reply,          /* Returns reply; ignored */
	long       handlerRefCon)   /* Application-defined parameter; ignored */
{
#pragma unused(reply,handlerRefCon)
	OSErr error;

	/* quit AE has no parms, but check in case the client requires any */
	error = DoneRequiredParams( quitAppleEvent );
	if (error == noErr)
		/* No extra parameters; handle Quit command */
		DoQuit();

	return error;
}


/******************************************************************************\
* DoneRequiredParams - Done processing required params; OK?
*
* DoneRequiredParams checks to see if the AppleEvent specified by the
* anAppleEvent parameter has any required parameters that we havent yet
* processed.  If there arent any left, then noErr is returned.  If there are
* required parameters that havent been processed yet, then errAEEventNotHandled
* is returned.  If any other errors occur, then that error code is returned.
\******************************************************************************/

static OSErr DoneRequiredParams(
	AppleEvent *anAppleEvent) /* AppleEvent being checked */
{
	DescType typeCode;   /* Type of AppleEvent attribute found; ignored */
	Size     actualSize; /* Actual size of parameters; ignored */
	OSErr    error;

	/* Are there any required parameters in AppleEvent we didnt process? */
   	error = AEGetAttributePtr(
   			anAppleEvent,
			keyMissedKeywordAttr,
			typeWildCard,
			/*<*/&typeCode,
			nil,
			0,
			/*<*/&actualSize );
	if (error == errAEDescNotFound)
		/* No required parameters left, so no error */
		error = noErr;
	else if (error == noErr)
		/* There was at least one required parameter we didnt process */
		error = errAEEventNotHandled;

	return error;
}


/******************************************************************************\
* Public: DoQuit
*
* Each open window is checked to see what kind it is, and then it is closed
* appropriately.
\******************************************************************************/

void DoQuit()
{
	WindowPtr aWindow; /* Pointer to each window in the window list */

	aWindow = FrontWindow();

	/* Keep closing a window until there are no windows left */
	while (aWindow != nil)
	{
		DisposeWindow( aWindow );
		aWindow = FrontWindow();
	}

	/* Tell the main event loop that were done */
	gQuitting = true;
}


/******************************************************************************\
* Public: DoUpdateEvt
*
* As new kinds of windows are added to this application, this routine will have
* to be able to detect the new kind of window and dispatch to the routine that
* handles update events in that kind of window.
\******************************************************************************/

void DoUpdateEvt(
	EventRecord *anEvent) /* Update event */
{
	WindowPtr eventWindow; /* Pointer to the window to update */

	/* Get a pointer to the window that needs an update */
	eventWindow = (WindowPtr)anEvent->message;

	/* Update the window that needs it */
	SetPort( eventWindow );
	BeginUpdate( eventWindow );
	EndUpdate( eventWindow );
}


/******************************************************************************\
* Public: ShowAlert
*
* To position the alert before its displayed, the ALRT resource is loaded
* before its displayed, and its boundsRect is put into the proper position
* through the CenterScreenRect routine.  Because this modifies the ALRT
* resource in memory and because ALRT resources are normally purgeable, it must
* be made unpurgeable until the alert is dismissed.
\******************************************************************************/

short ShowAlert(
	short alertType,    /* Type of alert to display */
	short buttonOption, /* Button options for alert */
	short messageClass, /* Class of message to display in alert */
	short messageIndex) /* Index of message to display in alert */
{
	Str255     aMessage;    /* Contents of message to place in alert */
	AlertTHndl alertHandle; /* Handle to the 'ALRT' resource for this alert */
	Rect       alertRect;   /* gxRectangle of alert */
	short      itemHit;     /* Item number of clicked item */
	SignedByte savedState;  /* Saves state alertHandle */
	Point      position;    /* Global coord of top left of alert window */

	/* Put the specified message into the dialog parameter text */
	if (messageIndex != 0)
	{
		GetIndString( /*<*/aMessage, messageClass, messageIndex );
		ParamText( aMessage, "\P", "\P", "\P" );
	}

	/* Show the stop alert */
	InitCursor();

	/* Center and display the alert */
	alertHandle = (AlertTHndl)Get1Resource( 'ALRT', buttonOption );
	if (alertHandle != nil)
	{
		/* Must make alert unpurgeable because were modifying it in memory */
		savedState = HGetState( (Handle)alertHandle );
		HNoPurge( (Handle)alertHandle );

		/* Put the gxRectangle of the alert into alert position */
		alertRect = (**alertHandle).boundsRect;
		position = PositionScreenRect( &alertRect, kMainScreenPos, kAlertPos,
				nil, 0, 0 );
		OffsetRect( /**/&alertRect, position.h - alertRect.left,
				position.v - alertRect.top );
		(**alertHandle).boundsRect = alertRect;

		/* Present the alert */
		if (alertType == kGenericAlert)
			itemHit = Alert( buttonOption, nil );
		else if (alertType == kNoteAlert)
			itemHit = NoteAlert( buttonOption, nil );
		else if (alertType == kCautionAlert)
			itemHit = CautionAlert( buttonOption, nil );
		else if (alertType == kStopAlert)
			itemHit = StopAlert( buttonOption, nil );

		/* Restore the state of alertHandle */
		HSetState( (Handle)alertHandle, savedState );
	}

	return itemHit;
}


/******************************************************************************\
* Public: TrapExists
*
* Traps with trap word of $AAXX or $ABXX are treated as being identical to $A8XX
* and $A9XX if gxColor QuickDraw isnt implemented, so we check for this case
* first.  We explicitly check to see if the trap word has a value greater than
* the largest possible trap word on this machine.  If it is, then the trap is
* automatically considered to be unimplemented.
\******************************************************************************/

Boolean TrapExists(
	short theTrap) /* Trap word being tested */
{
	TrapType theTrapType; /* The trap type of theTrap */

	/* If its a CQD trap but the trap table is too small, assume unimp. */
	theTrapType = GetTrapType( theTrap );
	if ((theTrapType == ToolTrap) && ((theTrap &= 0x07FF) >=
			MaxToolboxTrap()))
		theTrap = _Unimplemented;

	/* Return true if trap is implemented */
	return NGetTrapAddress( _Unimplemented, ToolTrap ) !=
			NGetTrapAddress( theTrap, theTrapType );
}


/******************************************************************************\
* Private: MaxToolboxTrap - Determine max trap number available
*
* Depending upon whether gxColor QuickDraw is implemented, the maximum trap number
* is either $0200 or $0400.  This routine tests to see which is the case by
* testing _InitGraf (trap $A86E) against trap $AA6E.  If the trap table is the
* bigger one, trap $AA6E always points either to Unimplemented or some other
* trap.  If the trap table is the smaller one, then $AA6E evaluates to the same
* trap as _InitGraf.  by comparing the address of the InitGraf trap against the
* address of the $AA6E trap, we can determine whether the trap table is the
* bigger one or the smaller one.  If we have the bigger trap table, then $0400
* is returned.  If its the smaller trap table, then $0200 is returned.
*
* Some of you might think that its be a lot simpler by testing to see whether
* gxColor QuickDraw is implemented or not.  Actually, I kind of think so too, but
* there are changes to the system software version 7.0 trap dispatcher that
* might make that test invalid, so I dont know.  I didnt write this, I just
* use it.
\******************************************************************************/

static short MaxToolboxTrap()
{
	if (NGetTrapAddress( _InitGraf, ToolTrap ) ==
			NGetTrapAddress(0xAA6E, ToolTrap))
		return 0x0200;
	else
		return 0x0400;
}


/******************************************************************************\
* Private: GetTrapType - Determine whether a trap is OS trap or Toolbox trap
*
* This routine tests to see whether the trap specified by theTrap is an
* operating system trap or a toolbox trap.  Operating system trap words begin
* with $A0 or $A1 while toolbox traps begin with $A8, $A9, $AA, or $AB.  By
* simply testing bit 11 of the trap word, we can determine which kind of trap
* theTrap is.  If its an operating system trap, then OSTrap is returned.  If
* its a toolbox trap, then ToolTrap is returned.
\******************************************************************************/

static TrapType GetTrapType(
	short theTrap) /* Trap word being tested */
{
	/* OS traps start with A0 or A1, Toolbox traps with A8, A9, AA, or AB */
	if ((theTrap & 0x0800) == 0)
		return OSTrap;
	else
		return ToolTrap;
}


/******************************************************************************\
* Public: EqualFSSpec
*
* To compare names, Im using EqualString with no case sensitivity, but with
* sensitivity to diacriticals.  This isnt usually the recommended way of
* comparing strings, because the Script Manager has routines for comparing
* strings in a more sophisticated, localizable way.  But the File Manager uses
* _CmpString, which is the assembly language equivalent of EqualString, so
* thats the way I must do things here.
\******************************************************************************/

Boolean EqualFSSpec(
	FSSpecPtr spec0, /* First FSSpec being compared */
	FSSpecPtr spec1) /* Second FSSpec being compared */
{
	return (spec0->vRefNum == spec1->vRefNum) &&
	       (spec0->parID == spec1->parID) &&
	       (EqualString(spec0->name, spec1->name, !kCaseSens, kDiacSens));
}

