/**
 --		B3 QuickDraw GX shell.c
 --
 --		This file is a shell that can be used to build "new" Graphics applications.  
 --		It contains all of the required calls to use the "new" Graphics routines and 
 --		QuickDraw (i.e. windows) together. It will put up one window. You "quit" 
 --		the application by clicking in the close bix.  This shell does not use a menu.
 --		
 -- 	The application is expected to supply the following functions which are called
 --		by this shell:
 --
 --			void DoSetup();
 --			void DoInitialization(WindowPtr);
 --			void DoDraw(WindowPtr);
 --			void DoDispose(WindowPtr);
 --			void DoClick(WindowPtr);
 --			void DoIdle(WindowPtr);
 --         void DoTeardown();
 --
 --
 --     Change History: 		
 --
 --		3/90	New
 --		6/91	Updated the shell to reflect the changes in "Graphics" v1.0d21.2.  PLA
 --		6/92	Made the following variables global: gDebugging, gGiveMeValidation,gGraphicsHeapSize. 
 --				See the comments for detail in this file.
 --		6/92	Added printing guts, mangled and reorganized.  DMH
 --		9/93	Updated files to work with the 2 "GXified" interface files.   PLA
 --		9/93	Added override for gxPrintingEvent to handle update events.	   DMH
 --		9/93	Added GXUpdateJob for resume events.	DMH
 --	   12/93	Extensive changes made for clarity, shell-like ability, cleanliness.  MD
 --
 --		 Apple Computer, Inc. 1990 - 1993  All rights reserved
 --								
**/


#include <Desk.h>
#include <Events.h>
#include <Fonts.h>
#include <Windows.h>
#include <Memory.h>
#include <ToolUtils.h>
#include <Menus.h>
#include <Quickdraw.h>

#include "PrintingManager.h"
#include "graphics toolbox.h"
#include "graphics routines.h"
#include "graphics libraries.h"
#include "graphics debugging.h"
#include "B3 QuickDraw GX shell.h"

Boolean			gQuitting;
long			gSleep = 0;

/*------ main ----------------------------------------------------------------------------------*/

void main()
{		
	CursHandle			theCurs; 
	Handle				menuBar;
	gxGraphicsClient 	client;
	OSErr				err;
	WindowPtr			wind;
	
	// Generic heap initialization.
	
	MaxApplZone(); 
	MoreMasters(); MoreMasters(); MoreMasters(); 
	MoreMasters(); MoreMasters(); MoreMasters(); 

	// Before we do anything else, we'll call the DoSetup routine so we can initialize any
	// global variables or behaviors.  This routine would be a good time to change the
	// gGraphicsHeapSize variable before we make the new graphics client next.
	
	DoSetup();
	
	
	// The GXNewGraphicsClient routine defines the graphics heap size.  If you do not make this
	// call, the GX graphics engine will create this heap automatically.  How?  It will create
	// a heap which is a percentage of your application's ideal memory foot print.  This call
	// allows you to explicity define the ammount of memory used by the graphics system for
	// its graphics objects heap.

	client = GXNewGraphicsClient(nil, gGraphicsHeapSize * 1024, 0L);

	// If gDebugging = TRUE, you will receive graphics library errors & notices will be posted.
	// This functionality will only work with the "debugging" version of QuickDraw GX.  If you
	// don't have the debugging version installed, these functions will not work. 

	if (gDebugging) {
		SetGraphicsLibraryErrors ();
		SetGraphicsLibraryNotices();	
	}

	// Set gGiveMeValidation to TRUE if you want run-time validation. As you increase the amount
	// of validation, The drawing speed will SLOW down due to all of the internal checking. 
		
	// gxPublicValidation will check parameters to public routines. For additional details
	// regarding the various levels of validation, see "Inside Macintosh: QuickDraw GX
	// Environment and Utilties."

	if (gGiveMeValidation) GXSetValidation(gxPublicValidation); 


	// Initialize the new graphics and printing environments.

	GXEnterGraphics();
	err = GXInitPrinting();

	// Initialize the other managers, if no errors occured.
 	
 	if (!err)
 	{
 		InitGraf(&qd.thePort);
		InitFonts();
		InitWindows();
		InitMenus();
		TEInit();
		InitDialogs(nil);
		InitCursor();

		theCurs = GetCursor(watchCursor);
		SetCursor(*theCurs);

		menuBar = GetNewMBar(rMenuBar); // Create the menu bar.

		if (menuBar)
		{
			gQuitting = false;
			SetMenuBar(menuBar);
			DisposeHandle(menuBar);
			AddResMenu(GetMHandle(mApple), 'DRVR'); // add Apple Menu items.
			DrawMenuBar();

 	// We initialize the CommonColors Library.  This will allow us to set the color of a
 	// shape by calling the SetShapeCommonColor function. We will need to call
 	// DisposeCommonColors when we leave, to clean up the world.

  			InitCommonColors();
	
			DoCreateNew();
			SetCursor(&qd.arrow);  
			
			while (!gQuitting)
				EventLoop();


	// Leaving.  Close all the windows so we get rid of any data we or GX created.  Then,
	// dispose of the common colors and exit the GX printing and graphics environment.

			while (wind = FrontWindow())
				DoDispose(wind);

			DisposeCommonColors();
		}

		GXExitPrinting();	// Close the new printing mgr. 
	}
	
	GXExitGraphics();		// Deallocate all of the default structures 
	GXDisposeGraphicsClient(client);
	DoTeardown();			// Dispose of any global variables we made in DoSetup.
}


/*------ MyPrintingEventOverride ----------------------------------------------------------------*/
// Override for GXPrintingEvent.  It allows us to update our windows
// when the moveable modal printing dialogs are moved.  Keep this in your
// main segment, which will always be loaded when GX tries to call the routine.

OSErr MyPrintingEventOverride(EventRecord *anEvent, Boolean filterEvent)
{

// Handle events in whatever way is appropriate.  MyDoEvent is our
// generic event handler.

	if (!filterEvent) MyDoEvent(anEvent);

	return noErr;
}


/*------ EventLoop ------------------------------------------------------------------------------*/

void EventLoop()
{
	EventRecord		theEvent;

	if (WaitNextEvent(everyEvent, &theEvent, gSleep, nil))
		MyDoEvent(&theEvent);
	else
		DoIdle(FrontWindow());
}



/*------ IsAppWindow ----------------------------------------------------------------------------*/
//
// This routine looks to make sure a window pointer is not nil and that the window it points
// to is of kind userKind.  If it is, it returns true, else it returns false.
//

Boolean IsAppWindow(WindowPtr wind)
{

	return	((((WindowPeek) wind)->windowKind == userKind) && (wind != nil));
}


/*------ MyDoEvent ------------------------------------------------------------------------------*/


void MyDoEvent(EventRecord *theEvent)
{
	char			key;
	WindowPtr 		whichWindow;		// temporarily used to hold 
	GrafPtr			oldPort;

	switch(theEvent->what)
	{					
		case mouseDown:
		switch (FindWindow(theEvent->where, &whichWindow)) {
			case inSysWindow:
				SystemClick(theEvent, whichWindow);
			break;
					
			case inDrag:
				if (IsAppWindow(whichWindow))
				    DragWindow(whichWindow, theEvent->where, &qd.screenBits.bounds);
			break;

			case inGoAway:
				if (IsAppWindow(whichWindow))
				{
					if (TrackGoAway(whichWindow, theEvent->where))
						DoDispose(whichWindow);
				}
			break;
			
			case inContent:
				if ((whichWindow != FrontWindow()) && (IsAppWindow(whichWindow)))
				 	SelectWindow(whichWindow);
			break;
			case inMenuBar:
				DoMenuCommand(MenuSelect(theEvent->where));
			break;

		}

		case keyDown:
		case autoKey:
			key = theEvent->message & charCodeMask;
			if (theEvent->modifiers & cmdKey)
				if (theEvent->what == keyDown)
					DoMenuCommand(MenuKey(key));
			break;


		case updateEvt:
			whichWindow = (WindowPtr)theEvent->message;
			if (IsAppWindow(whichWindow))
			{
		        GetPort(&oldPort);
				SetPort(whichWindow);
		        BeginUpdate(whichWindow);
				DoDraw(whichWindow, true);
			    EndUpdate(whichWindow);
		        SetPort(oldPort);
		      }
			break;
		
		case activateEvt:
		
			break;
		
		case osEvt:
			switch ((unsigned long) theEvent->message >> 24) {	//  high byte of message  
			 	case suspendResumeMessage:	//  suspend/resume is also an activate/deactivate 
					if ((theEvent->message & resumeFlag) == 0)
					   gSleep = 80;			// we are headed to the background, so slow down... 
					else
					{
					   gSleep = 0;			// we are headed to the foreground, so speed up...  

// On a resume event, we need to call GXUpdateJob on all of our documents'
// jobs.  This is important because the user may have just changed
// something which affects our jobs (like the size of the paper in the
// printer).
//
// Since our application stores our document references in the refCon fields
// of our documents' windows, we just loop through every one of our windows,
// extract our document pointers and update the associated jobs.

						whichWindow = FrontWindow();
						
						while (whichWindow != nil)
						{
							if (IsAppWindow(whichWindow))
								GXUpdateJob(GetDocJob(whichWindow));
							
							whichWindow = (WindowPtr) ((WindowPeek) whichWindow)->nextWindow;
						}
					}
				break;
			}
			break;
		
	}
}

