/*
	Hindi Layout Sample.c
	
	This file contains a sample of a layout shape using Hindi text.  It
	requires the Pratiksha font if you don't want to see a lot of strange characters.
	
	1992 - 1993  Apple Computer, Inc.
	All rights reserved.
	
	Updated by Matt Deatherage.
	Originally by Eric Mader, after Mike Fairman, Oliver Steele, et. al.
*/

#include <StdIO.h>

#include <resources.h>
#include <events.h>
#include <memory.h>
#include <windows.h>
#include <toolutils.h>

#include "graphics routines.h"
#include "graphics toolbox.h"
#include "layout routines.h"
#include "graphics libraries.h"
#include "layout library.h"
#include "graphics debugging.h"
#include "PrintingMessages.h"
#include "PrintingManager.h"

#include "B3 QuickDraw GX shell.h"
#include "Hindi Layout Sample.h"


/*------ Global variables for the shell ----------------------------------------------------*/

// Set up the title and size of the window

Str255 		gWindowTitle;
Rect 		gWindowQDRect;

// If gDebugging = TRUE, graphics library errors and notices will be posted.  This
// functionality will only work with the "debugging" version of QuickDraw GX.
// If the debugging version is not installed, nothing bad will happen, but these
// functions will not work. 

Boolean		gDebugging = true;

// Set  "gGiveMeValidation" to TRUE if you want receive run-time validation.

Boolean		gGiveMeValidation = true;


// gGraphicsHeapSize sets the size of the graphics heap created by calling the
// GXNewGraphicsClient routine in main () within B3 QuickDraw GX shell.c.  You can determine
// the amount of graphics heap required by using GraphicsBug.  I'm giving it 600K,
// since we need abunch if we have several windows open.

long		gGraphicsHeapSize = 600;


/*------ DoSetup ---------------------------------------------------------------------------*/
//
//	Here's where we initialize any global variables our application needs.  We have two of
//	them -- gWindowTitle and gWindowQDRect.  We obtain both of them from our resource fork,
//	so we don't have to recompile to change such things for international markets, etc.

void DoSetup(void)
{

	Handle	windRectHandle;

// Get the title and rectangle for our window.  These are in our resource fork -- the
// title as a string in 'STR#' resource number 'rWindTitleString' and the rectangle as
// 'RECT' resource number 'rWindRectangle'.  These are all defined in the header file
// "B3 QuickDraw GX shell.h".

	GetIndString(gWindowTitle, rWindTitleString, rWindTitleStringIndex);
	windRectHandle = GetResource('RECT',rWindRectangle);
	gWindowQDRect = *(RectPtr)*windRectHandle;
	ReleaseResource(windRectHandle);
}

/*------ DoInitialization ------------------------------------------------------------------*/
//
//	In this function we create and initialize the the private document structure for
//  a new window.  This structure contains the print job and the shape which is drawn
//  in the window.  We store this data in a handle and hang it off the window's refCon
//  field for easy retrieval.  By doing this, rather than using globals, we can create
//  many windows containing unique print jobs and shapes. 

OSErr DoInitialization(wind)
WindowPtr wind;
{
	OSErr	err = noErr;
	gxJob	docJob;
	gxShape	docPage;
	TH_Doc	windDoc;


// Create the page shape. We set the unique items attribute to make sure that each item
// added to the picture has a unique reference. If this attribute was not set, we would
// not see all copies of anything we add to the shape multiple times -- we'd just see
// the last version added.		

	docPage = GXNewShape(gxPictureType);
	GXSetShapeAttributes(docPage, gxUniqueItemsShape);
	
	
// Create a print job for this document.  This will be the same as the system default until
// the user goes through the dialogs for Page Setup or Print

	err = GXNewJob(&docJob);
	
	
// If there are no errors, create a handle the size of our document structure and store
// the print job and page shape in it.  Store the handle in the window's refCon field so
// that we can get at it.  (Note that the utility routines "GetDocJob" and "GetDocShape"
// can be used to do this easily.

	if (!err)
	{
		windDoc = (TH_Doc) NewHandleClear(sizeof(T_Doc));

		if (!windDoc)
			err = MemError();
		else
		{
			(*windDoc)->docJob = docJob;
			(*windDoc)->docPage = docPage;
			((WindowPeek) wind)->refCon = (long) windDoc;
		}

// Now install our application override for PrintingEvent so that we can
// support the new movable-modal printing dialog boxes.

		GXInstallApplicationOverride(docJob, gxPrintingEvent, MyPrintingEventOverride);

	}

	return err;
}


/*------ DoDraw ----------------------------------------------------------------------------*/
//
//	Draw the contents of the window.
// 
void DoDraw(wind)
WindowPtr wind;
{
	 GXDrawShape (GetDocShape(wind));
}


/*------ CreateSampleLayoutImage -----------------------------------------------------------*/
//
//	This function creates our layout and highlight shapes and adds them to the 
//  window's page shape.
//
void CreateSampleLayoutImage(WindowPtr wind)
{

	gxShape layout;						// the layout shape we build
	gxShape highlight;					// the highlight shape for the layout shape
	gxShape thePage;					// this window's document shape
	gxRunControls runControls;			// run controls for the layout shape
	gxLayoutOptions layoutOptions;		// options for the layout shape

// The following is "Hindi" in Hindi: 
// (This assumes the standard Hindi character set for Macintosh, by the way)
	
	static char text[] = {0xE3, 0xE7, 0xD0, 0xF7, 0xCE, 0xE8, 0};
	
	gxPoint posn;						// the position of the layout shape


// make default gxLayoutOptions and gxRunControls structures

	InitializeLayoutOptions (&layoutOptions);
	InitializeRunControls (&runControls);
	
	
// Position the layout half way down the left edge of the window. Set 
// the layout's width to the window's width and set the flushness to 1/2, this
// will cause the layout to center in the window.  Note that gWindowQDRect is
// not in fixed coordinates, so we have to make it fixed to use it.

	layoutOptions.width = ff(gWindowQDRect.right - gWindowQDRect.left);
	layoutOptions.flush = fract1/2;

	posn.x = 0;
	posn.y = ff((gWindowQDRect.bottom - gWindowQDRect.top) / 2);
	

// Create the layout we'll use.  NewSingleLayout is a library routine found in
// layout library.c which builds a layout based on one style.  But first,
// this important message:

// When I was converting this sample to actually print, I found this strange
// problem that the highlight shape, while appearing correctly on the screen,
// was always too far to the right when printed -- sometimes overlapping parts
// of glyphs that shouldn't be highlighted.  An investigation was launched.
//
// It turns out that highlight shapes are pretty much always 72 DPI, because
// they're polygons calculated from the metrics of the layout shape when the
// highlight shape is created.  If you then scale the layout shape by
// transforming it to 300 DPI or so (like a printer driver would do), the
// highlight shape has the same geometry while hinting of the glyphs may have
// changed the way the layout shape looks.  They've moved a little bit, perhaps,
// and therefore the highlight shape isn't in the right place anymore.
//
// If this is for your on-screen work, you can recreate the highlight shape in
// a zoomed viewPort, but you don't get to make the printer driver do that for
// you.  So, instead, for each style run we use, we add the gxNoMetricsGridText
// attribute.  This prevents QuickDraw GX from using hinting, so the glyphs always
// scale the same way the highlight shape does, and all is well.
//
// (At first, I tried to fix this by adding gxNoMetricsGridShape to the layout
// _shape's_ attributes, but that's not the right idea.  The attributes in the
// shape's style affect any part of the layout that doesn't have a style of its
// own, including any parts of the text for which the entry in the style array
// is nil.  Since we're actually applying styles over the layout here, the
// gxNoMetricsGridText must be set in those styles, not in the shape's style.)
//
// This does mean that your glyphs won't get hinting at low resolutions and
// small point sizes where it might be desirable -- but it's really only a problem
// if you're trying to print a highlight shape, which doesn't happen very often.
// In fact, I can't really think of a legitimate reason to want to do it, except
// maybe in sample code.
	
	layout = NewSingleLayout(
				text, 							// the text for the layout
				(char *) "\pPratiksha Medium", 	// the font name to use
				ff(200), 						// the point size to use
				&layoutOptions, 				// our default layout options
				&posn, 							// the shape's position
				gxNoMetricsGridText, 			// attributes to make printing work
				&runControls,					// our default run controls
				nil, 							// the run features array (none)
				0, 								// count of zero features = 0
				nil);							// no style run overrides
	
// Retrieve the page shape so we can add to it.

	thePage = GetDocShape(wind);
	
// Add the layout to the window's picture shape.  AddToShape is in shape library.c.

	AddToShape(thePage, layout);

// Make a highlight shape to highlight the layout from character 5 to character 20.
// This will go from the level 0 English text into the level 2 English text
// which is nested in the level 1 Hindi text, and will produce a highlight shape
// consisting of three disjoint rectangles because the Hindi text is displayed
// right-to-left.  SetShapeCommonTransfer is in transferMode library.c, and
// SetShapeCommonColor is in color library.c.

	highlight = GXGetLayoutHighlight(
	layout,								// the layout shape to get the highlight from
	1, 5, 								// starting and ending byte offsets
	gxHighlightAverageAngle, 			// highlight type -- use the average angle
	nil);								// a shape to use if we have one around (we don't)
	
	SetShapeCommonTransfer (highlight, gxHighlightMode);
	SetShapeCommonColor (highlight, gxWhite);

	
	AddToShape(thePage, highlight);
	

// Now that both shapes are added to the picture, dispose of them

	GXDisposeShape(highlight);
	GXDisposeShape(layout);
	

// Invalidate the window's portRect so that everything gets updated.
	
	SetPort(wind);
	InvalRect(&wind->portRect);
}


/*------ DoCreateNew -----------------------------------------------------------------------*/
//
//	This routine is called when a window needs to be created.
//
OSErr DoCreateNew(void)
{
	OSErr		err;
	WindowPtr	wind;
	
// Create a window, attach a default gxViewPort to it, create and iInitialize our
// private data for it, and add a sample image to its page shape.

	wind = NewWindow(nil, &gWindowQDRect, gWindowTitle, true, noGrowDocProc,
	                 (WindowPtr)-1L, true, 0L);
	
	if (!wind)
		err = MemError();
	else
	{
		GXIgnoreGraphicsNotice(transform_already_set);
		SetDefaultViewPort(GXNewWindowViewPort(wind));			
		GXPopGraphicsNotice();
		err = DoInitialization(wind);
	}
	
	CreateSampleLayoutImage(wind);
	return err;
}


/*------ DoDispose -------------------------------------------------------------------------*/
//
//	This routine is called when a window needs to be disposed of.
//
void DoDispose(wind)
WindowPtr wind;
{
	TH_Doc	doc;
	
// You should always dispose of your GX graphics objects before tossing your window.
// Why?  It's generally good form and this approach guarantees that everything is
// disposed.  If you had not disposed of everything, the call to DisposeWindow should
// dispose of the objects. If you are running the debugging version of QuickDraw GX
// with notices set, you will receive a notice that you had not disposed of everything.
// You can turn notices on in this file by setting gDebugging = TRUE (above).
	
	doc = (TH_Doc)  GetWRefCon(wind);	// Remember, this is where we stored our private data.
	
	GXDisposeShape(GetDocShape(wind)); 	// Dispose of this doc's shape.
	GXDisposeJob(GetDocJob(wind));		// Dispose of this doc's print job.
	DisposHandle((Handle) doc);			// Dispose of our private data.

   	DisposeWindow(wind);				// Dispose of the window.
}


/*------ DoIdle ----------------------------------------------------------------------------*/
//
//	This routine is called to do things while we're idling through the event loop.
//
void DoIdle(WindowPtr wind)
{
#pragma unused (wind)
}

/*------ DoTearDown ------------------------------------------------------------------------*/
//
//	This routine is called just before we quit to remove anything persistent that might have
//  been setup by DoSetup().
//
void DoTeardown(void)
{
}

