/*
	Spiral Layout Sample.c
	
	This file contains the calls that implement the spiraling layout sample.
	
	1992 - 1993  Apple Computer, Inc.
	All rights reserved.
*/

#include <StdIO.h>

#include <resources.h>
#include <events.h>
#include <memory.h>
#include <windows.h>
#include <toolutils.h>

#include "graphics routines.h"
#include "graphics toolbox.h"
#include "layout routines.h"
#include "graphics libraries.h"
#include "layout library.h"
#include "graphics debugging.h"
#include "PrintingMessages.h"
#include "PrintingManager.h"

#include "B3 QuickDraw GX Shell.h"


/*------ Global variables for the shell ----------------------------------------------------*/

// Set up the title and size of the window

Str255 		gWindowTitle;
Rect 		gWindowQDRect;

// If gDebugging = TRUE, graphics library errors and notices will be posted.  This
// functionality will only work with the "debugging" version of QuickDraw GX.
// If the debugging version is not installed, nothing bad will happen, but these
// functions will not work. 

Boolean		gDebugging = true;

// Set  "gGiveMeValidation" to TRUE if you want receive run-time validation.

Boolean		gGiveMeValidation = true;


// gGraphicsHeapSize sets the size of the graphics heap created by calling the
// GXNewGraphicsClient routine in main () within B3 QuickDraw GX shell.c.  You can determine
// the amount of graphics heap required by using GraphicsBug.  I'm giving it 600K,
// since we need abunch if we have several windows open.

long		gGraphicsHeapSize = 600;


/*------ DoSetup ---------------------------------------------------------------------------*/
//
//	Here's where we initialize any global variables our application needs.  We have two of
//	them -- gWindowTitle and gWindowQDRect.  We obtain both of them from our resource fork,
//	so we don't have to recompile to change such things for international markets, etc.

void DoSetup(void)
{

	Handle	windRectHandle;

// Get the title and rectangle for our window.  These are in our resource fork -- the
// title as a string in 'STR#' resource number 'rWindTitleString' and the rectangle as
// 'RECT' resource number 'rWindRectangle'.  These are all defined in the header file
// "B3 QuickDraw GX shell.h".

	GetIndString(gWindowTitle, rWindTitleString, rWindTitleStringIndex);
	windRectHandle = GetResource('RECT',rWindRectangle);
	gWindowQDRect = *(RectPtr)*windRectHandle;
	ReleaseResource(windRectHandle);
}

/*------ DoInitialization ------------------------------------------------------------------*/
//
//	In this function we create and initialize the the private document structure for
//  a new window.  This structure contains the print job and the shape which is drawn
//  in the window.  We store this data in a handle and hang it off the window's refCon
//  field for easy retrieval.  By doing this, rather than using globals, we can create
//  many windows containing unique print jobs and shapes. 

OSErr DoInitialization(WindowPtr wind)
{
	OSErr		err = noErr;
	gxJob		docJob;
	gxShape		docPage;
	TH_Doc		windDoc;


// Create the page shape. We set the unique items attribute to make sure that each item
// added to the picture has a unique reference. If this attribute was not set, we would
// not see all copies of anything we add to the shape multiple times -- we'd just see
// the last version added.		

	docPage = GXNewShape(gxPictureType);
	GXSetShapeAttributes(docPage, gxUniqueItemsShape);
	
	
// Create a print job for this document.  This will be the same as the system default until
// the user goes through the dialogs for Page Setup or Print

	err = GXNewJob(&docJob);

	
// If there are no errors, create a handle the size of our document structure and store
// the print job and page shape in it.  Store the handle in the window's refCon field so
// that we can get at it.  (Note that the utility routines "GetDocJob" and "GetDocShape"
// can be used to do this easily.  Put our transform in there, too.

	if (!err)
	{
		windDoc = (TH_Doc) NewHandleClear(sizeof(T_Doc));

		if (!windDoc)
			err = MemError();
		else
		{
			(*windDoc)->docJob = docJob;
			(*windDoc)->docPage = docPage;
			
			SetWRefCon(wind, (long) windDoc);
		}

// Now install our application override for PrintingEvent so that we can
// support the new movable-modal printing dialog boxes.

		GXInstallApplicationOverride(docJob, gxPrintingEvent, MyPrintingEventOverride);

	}

	return err;
}


/*------ DoDraw ----------------------------------------------------------------------------*/
//
//	Draw the contents of the window.  The first parameter is the window to draw, and the
//  second parameter is true if we're updating an existing image.  If that's the case,
//  we don't want to change anything, but just draw what's already there.
// 
void DoDraw(WindowPtr wind, Boolean updating)
{
	 #pragma unused (updating)
	 GXDrawShape(GetDocShape(wind));
}


/*------ StrLength ------------------------------------------------------------------------*/
//
//	This function takes the length of a C-style string quickly without requiring an ANSI
//  library to be linked in.
//
static short StrLength(char *s)
{
	short len;

	for (len = 0; *s++ != 0; len++) ;
	return len;
}


/*------ CreateSampleLayoutImage -----------------------------------------------------------*/
//
//	This function creates all our spirals and adds them to the page shape.  We don't
//  do anything on idle in this sample anymore -- check out Circular Layout Sample
//  for an example of an animated layout sample.
//
void CreateSampleLayoutImage(WindowPtr wind)
{

	gxShape layout;						// the layout shape we build
	gxShape thePage;					// this window's document shape
	gxRunControls runControls;			// run controls for the layout shape
	gxLayoutOptions layoutOptions;		// options for the layout shape
	gxStyle timesStyle;					// a style record for a Times-based style
	gxStyle helveticaStyle;				// ditto for Helvetica
	gxStyle baghdadStyle; 				// ditto for Baghdad
	gxStyle hoeflerStyle; 				// ditto for Hoefler
	gxStyle textStyles[4];				// an array of text style for our text runs
	
// These are the four text runs for this layout shape, in pieces because they're
// in different languages and styles
	
	char *text1 = "Aetna ";
	
// The following is "Macintosh" in Arabic: 
// meem, alif, kaf,  noon, tah,  wau,  shin
// (This assumes the standard Arabic character set for Macintosh, by the way)
	
	static char text2[] = {0xE5, 0xC7, 0xE3, 0xE6, 0xCA, 0xE8, 0xD4, 0};
	
	char *text3 = " Office";
	char *text4 = 		" AWAY.";

	char *textRuns[4];					// an array of pointers to text runs

	short textLengths[4];				// an array of the lengths of each run
	short totalLength;					// the total length of the layout's text
	short level0 = 0;					// so we can take the address of it later
	
	gxPoint posn;						// the position of the layout shape
	
	gxColor ourColor;					// the color we'll use
		
	short	counter;					// loop variable
	gxTransform ourTransform;			// transform for our layout shape


// First, initialize the textRuns array to point to the four text runs
	
	textRuns[0] = text1;
	textRuns[1] = text2;
	textRuns[2] = text3;
	textRuns[3] = text4;
	
	
// Next, initialize the textLengths and levelRunLengths arrays.  StrLength() is
// in this file.

	textLengths[0] = StrLength (text1);
	textLengths[1] = StrLength (text2);
	textLengths[2] = StrLength (text3);
	textLengths[3] = StrLength (text4);
	
// totalLength is the length of all the text.

	totalLength = textLengths[0] + textLengths[1] + textLengths[2] + textLengths[3];
	

// make default gxLayoutOptions and gxRunControls structures

	InitializeLayoutOptions (&layoutOptions);
	InitializeRunControls (&runControls);
	
	
// Put the start of the layout in the center of the window

	posn.x = ff((gWindowQDRect.right - gWindowQDRect.left) / 2);
	posn.y = ff((gWindowQDRect.bottom - gWindowQDRect.top) / 2);
	

// Create the styles we'll use.  All styles are 24-point text.
// NewLayoutStyle is a library routine found in layout library.c.


	GXIgnoreGraphicsNotice(text_size_already_set);
	helveticaStyle = NewLayoutStyle(
		(char *) "\pHelvetica",				// the gxFontName for the style
		ff(12),								// the text size in fixed point
		0,									// gxTextAttributes
		&runControls,						// run controls (our default ones)
		nil,								// run features (none for this style)
		0,									// count of run features
		nil);								// style run overrides (none right now)
	GXPopGraphicsNotice();

	GXIgnoreGraphicsNotice(text_size_already_set);
	baghdadStyle = NewLayoutStyle(
		(char *) "\pBaghdad Plain",			// gxFontName
		ff(12),								// text size (fixed point)
		0,									// gxTextAttributes
		&runControls,						// run controls (our default ones)
		nil, 								// run features (none)
		0, 									// count of run features (none)
		nil);								// style run overrides (none)
	GXPopGraphicsNotice();
	
	GXIgnoreGraphicsNotice(text_size_already_set);
	timesStyle = NewLayoutStyle(
		(char *) "\pTimes Roman", 			// gxFontName
		ff(12), 							// text size (fixed point)
		0,									// gxTextAttributes
		&runControls, 						// run controls (our default ones)
		nil, 								// run features (none)
		0, 									// count of run features (none)
		nil);								// style run overrides (none)
	GXPopGraphicsNotice();
	
	GXIgnoreGraphicsNotice(text_size_already_set);
	hoeflerStyle = NewLayoutStyle(
		(char *) "\pHoefler Italic", 			// gxFontName
		ff(12), 							// text size (fixed point)
		0,									// gxTextAttributes
		&runControls, 						// run controls (our default ones)
		nil, 								// run features (none)
		0, 									// count of run features (none)
		nil);								// style run overrides (none)
	GXPopGraphicsNotice();

// Initialize the textStyles array so that each of the five runs has the right style

	textStyles[0] = helveticaStyle;
	textStyles[1] = baghdadStyle;
	textStyles[2] = hoeflerStyle;
	textStyles[3] = timesStyle;
	

// Setup complete!  Build the layout.

	layout = GXNewLayout(
		4,								// count of text runs
		textLengths,					// array of lengths of each run
		(void *) textRuns,				// array of pointers to the text
		4,								// count of style runs
		textLengths,					// array of the byte lengths of each run
		textStyles,						// array of the styles
		1,								// number of levels in the layout
		&totalLength,					// array of the lengths of each level
		&level0,						// array of the levels
		&layoutOptions,					// options (default)
		&posn);							// position of the layout shape
		
// Retrieve the page shape so we can add to it.

	thePage = GetDocShape(wind);
	
// Get the transform for our layout

	ourTransform = GXGetShapeTransform(layout);
	
// Initialize our color to red

	(void *) SetCommonColor(&ourColor, red);
	(void *) GXConvertColor(&ourColor, gxHSVSpace, nil, nil);	// convert to HSV

// Now, rotate the layout 15 degrees 24 times for a total of 360 degrees.
// Each rotation we'll also change the hue by 2730 and scale in both directions
// by 17/16.	

	for (counter = 0; counter <= 24; counter++)
	{
		GXSetShapeColor(layout, &ourColor);
		AddToShape(thePage, layout);
		ourColor.element.hsv.hue += 0x0AAA;
		GXRotateTransform(
			ourTransform,				// the transform to spin
			ff(15),						// how many degrees to rotate
			posn.x,						// the x center of rotation
			posn.y);					// the y center of rotation
		GXScaleTransform(
			ourTransform,				// the transform to scale
			(fixed)0x00011000,			// scale horizontally by 17/16
			(fixed)0x00011000,			// scale vertically by 17/16
			posn.x,						// the x center of scaling
			posn.y);					// the y center of scaling
	}

// Dispose of the shapes and styles we created

	GXDisposeShape(layout);
	GXDisposeStyle(helveticaStyle);	
	GXDisposeStyle(baghdadStyle);	
	GXDisposeStyle(timesStyle);	
	GXDisposeStyle(hoeflerStyle);	
	
// Invalidate the window's portRect so that everything gets updated.
	
	SetPort(wind);
	InvalRect(&wind->portRect);
}

/*------ DoCreateNew -----------------------------------------------------------------------*/
//
//	This routine is called when a window needs to be created.
//
OSErr DoCreateNew(void)
{
	OSErr		err;
	WindowPtr	wind;
	
// Create a window, attach a default gxViewPort to it, create and iInitialize our
// private data for it, and add a sample image to its page shape.

	wind = NewWindow(nil, &gWindowQDRect, gWindowTitle, true, noGrowDocProc,
	                 (WindowPtr)-1L, true, 0L);
	
	if (!wind)
		err = MemError();
	else
	{
		GXIgnoreGraphicsNotice(transform_already_set);
		SetDefaultViewPort(GXNewWindowViewPort(wind));			
		GXPopGraphicsNotice();
		err = DoInitialization(wind);
	}
	
	CreateSampleLayoutImage(wind);
	return err;
}


/*------ DoDispose -------------------------------------------------------------------------*/
//
//	This routine is called when a window needs to be disposed of.
//
void DoDispose(wind)
WindowPtr wind;
{
	TH_Doc	doc;
	
// You should always dispose of your GX graphics objects before tossing your window.
// Why?  It's generally good form and this approach guarantees that everything is
// disposed.  If you had not disposed of everything, the call to DisposeWindow should
// dispose of the objects. If you are running the debugging version of QuickDraw GX
// with notices set, you will receive a notice that you had not disposed of everything.
// You can turn notices on in this file by setting gDebugging = TRUE (above).
	
	doc = (TH_Doc)  GetWRefCon(wind);	// Remember, this is where we stored our private data.
	
	GXDisposeShape(GetDocShape(wind)); 	// Dispose of this doc's shape.
	GXDisposeJob(GetDocJob(wind));		// Dispose of this doc's print job.
	DisposHandle((Handle) doc);			// Dispose of our private data.

   	DisposeWindow(wind);				// Dispose of the window.
}


/*------ DoIdle ----------------------------------------------------------------------------*/
//
//	This routine is called to do things while we're idling through the event loop.
//
void DoIdle(WindowPtr wind)
{
#pragma unused (wind)
}

/*------ DoTearDown ------------------------------------------------------------------------*/
//
//	This routine is called just before we quit to remove anything persistent that might have
//  been setup by DoSetup().
//
void DoTeardown(void)
{
}

