/*
	Tag Layout Sample.c
	
	This file contains the calls that implement our tag layout sample.
	
	1992 - 1993  Apple Computer, Inc.
	All rights reserved.
*/

#include <StdIO.h>

#include <resources.h>
#include <events.h>
#include <memory.h>
#include <windows.h>
#include <toolutils.h>

#include "graphics routines.h"
#include "graphics toolbox.h"
#include "layout routines.h"
#include "layout feature constants.h"
#include "graphics libraries.h"
#include "layout library.h"
#include "graphics debugging.h"
#include "PrintingMessages.h"
#include "PrintingManager.h"

#include "B3 QuickDraw GX Shell.h"


/*------ Global variables for the shell ----------------------------------------------------*/

// Set up the title and size of the window

Str255 		gWindowTitle;
Rect 		gWindowQDRect;

// If gDebugging = TRUE, graphics library errors and notices will be posted.  This
// functionality will only work with the "debugging" version of QuickDraw GX.
// If the debugging version is not installed, nothing bad will happen, but these
// functions will not work. 

Boolean		gDebugging = true;

// Set  "gGiveMeValidation" to TRUE if you want receive run-time validation.

Boolean		gGiveMeValidation = true;


// gGraphicsHeapSize sets the size of the graphics heap created by calling the
// GXNewGraphicsClient routine in main () within B3 QuickDraw GX shell.c.  You can determine
// the amount of graphics heap required by using GraphicsBug.  I'm giving it 600K,
// since we need abunch if we have several windows open.

long		gGraphicsHeapSize = 600;


/*------ DoSetup ---------------------------------------------------------------------------*/
//
//	Here's where we initialize any global variables our application needs.  We have two of
//	them -- gWindowTitle and gWindowQDRect.  We obtain both of them from our resource fork,
//	so we don't have to recompile to change such things for international markets, etc.

void DoSetup(void)
{

	Handle	windRectHandle;

// Get the title and rectangle for our window.  These are in our resource fork -- the
// title as a string in 'STR#' resource number 'rWindTitleString' and the rectangle as
// 'RECT' resource number 'rWindRectangle'.  These are all defined in the header file
// "B3 QuickDraw GX shell.h".

	GetIndString(gWindowTitle, rWindTitleString, rWindTitleStringIndex);
	windRectHandle = GetResource('RECT',rWindRectangle);
	gWindowQDRect = *(RectPtr)*windRectHandle;
	ReleaseResource(windRectHandle);
}

/*------ DoInitialization ------------------------------------------------------------------*/
//
//	In this function we create and initialize the the private document structure for
//  a new window.  This structure contains the print job and the shape which is drawn
//  in the window.  We store this data in a handle and hang it off the window's refCon
//  field for easy retrieval.  By doing this, rather than using globals, we can create
//  many windows containing unique print jobs and shapes. 

OSErr DoInitialization(WindowPtr wind)
{
	OSErr	err = noErr;
	gxJob	docJob;
	gxShape	docPage;
	TH_Doc	windDoc;


// Create the page shape. We set the unique items attribute to make sure that each item
// added to the picture has a unique reference. If this attribute was not set, we would
// not see all copies of anything we add to the shape multiple times -- we'd just see
// the last version added.		

	docPage = GXNewShape(gxPictureType);
	GXSetShapeAttributes(docPage, gxUniqueItemsShape);
	
	
// Create a print job for this document.  This will be the same as the system default until
// the user goes through the dialogs for Page Setup or Print

	err = GXNewJob(&docJob);
	
	
// If there are no errors, create a handle the size of our document structure and store
// the print job and page shape in it.  Store the handle in the window's refCon field so
// that we can get at it.  (Note that the utility routines "GetDocJob" and "GetDocShape"
// can be used to do this easily.

	if (!err)
	{
		windDoc = (TH_Doc) NewHandleClear(sizeof(T_Doc));

		if (!windDoc)
			err = MemError();
		else
		{
			(*windDoc)->docJob = docJob;
			(*windDoc)->docPage = docPage;
			SetWRefCon(wind, (long) windDoc);
		}

// Now install our application override for PrintingEvent so that we can
// support the new movable-modal printing dialog boxes.

		GXInstallApplicationOverride(docJob, gxPrintingEvent, MyPrintingEventOverride);

	}

	return err;
}


/*------ DoDraw ----------------------------------------------------------------------------*/
//
//	Draw the contents of the window.  The first parameter is the window to draw, and the
//  second parameter is true if we're updating an existing image.  If that's the case,
//  we don't want to change anything, but just draw what's already there.
// 
void DoDraw(WindowPtr wind, Boolean updating)
{
	 #pragma unused (updating)
	 GXDrawShape (GetDocShape(wind));
}


/*------ StrLength ------------------------------------------------------------------------*/
//
//	This function takes the length of a C-style string quickly without requiring an ANSI
//  library to be linked in.
//
static short StrLength(char *s)
{
	short len;

	for (len = 0; *s++ != 0; len++) ;
	return len;
}


/*------ CreateSampleLayoutImage -----------------------------------------------------------*/
//
//	This function creates  primitive shapes and adds them to the window's page shape.
//
void CreateSampleLayoutImage(WindowPtr wind)
{

	gxShape layout1, layout2, layout3, layout4;
										// the four layout shapes we build
	gxShape thePage;					// this window's document shape
	gxRunControls runControls;			// run controls for the layout shape
	gxLayoutOptions layoutOptions;		// options for the layout shape

// The following three run features change the cases of our text as described
// in Inside Macintosh: QuickDraw GX Typography.

	static gxRunFeature lowerCaseFeatures[] = 
			{{letterNumberCaseType, letterNumberCaseAllLowerSelector}};
	static gxRunFeature upperCaseFeatures[] = 
			{{letterNumberCaseType, letterNumberCaseAllUpperSelector}};
	static gxRunFeature toggleCaseFeatures[] = 
			{{letterNumberCaseType, letterNumberCaseSmallCapsSelector}};

	char *text = "Th Qk Brow Fx Jmpd vr Th Lz Dgs ffic Aetn.";

	gxPoint posn;						// the position of the layout shape


// make default gxLayoutOptions and gxRunControls structures

	InitializeLayoutOptions (&layoutOptions);
	InitializeRunControls (&runControls);
	
	
// This time, we'll set the layout width to 8 inches, fully justified.  We'll
// center that in the window.  Since we're drawing four lines 30 points apart,
// we'll put the first two lines 15 and 45 points above the center of the window,
// and the second two 15 and 45 points below.

	layoutOptions.width = ff(8 * 72);
	layoutOptions.just = fract1;

	posn.x = (ff(gWindowQDRect.right - gWindowQDRect.left) - layoutOptions.width) / 2;
	posn.y = ff((gWindowQDRect.bottom - gWindowQDRect.top) / 2) - ff(45);
	

// Create the layouts we'll use.  NewSingleLayout is a library routine found
// in layout library.c to create a layout based on one style.

	layout1 = NewSingleLayout(
				text, 							// the text for the layout
				(char *) "\pHoefler Plain", 	// the font name to use
				ff(18), 						// the point size to use
				&layoutOptions, 				// our default layout options
				&posn, 							// the shape's position
				0, 								// attributes (none)
				&runControls,					// our run controls
				nil, 							// the run features array (none)
				0, 								// count of zero features = 0
				nil);							// no style run overrides
				
	posn.y += ff(30);
	
	layout2 = NewSingleLayout(
				text, 							// the text for the layout
				(char *) "\pHoefler Plain", 	// the font name to use
				ff(18), 						// the point size to use
				&layoutOptions, 				// our default layout options
				&posn, 							// the shape's position
				0, 								// attributes (none)
				&runControls,					// our run controls
				lowerCaseFeatures, 				// the run features array (none)
				1, 								// count of one feature = 1
				nil);							// no style run overrides
				
	posn.y += ff(30);
	
	layout3 = NewSingleLayout(
				text, 							// the text for the layout
				(char *) "\pHoefler Plain", 	// the font name to use
				ff(18), 						// the point size to use
				&layoutOptions, 				// our default layout options
				&posn, 							// the shape's position
				0, 								// attributes (none)
				&runControls,					// our run controls
				upperCaseFeatures, 				// the run features array (none)
				1, 								// count of one feature = 1
				nil);							// no style run overrides
				
	posn.y += ff(30);
	
	layout4 = NewSingleLayout(
				text, 							// the text for the layout
				(char *) "\pHoefler Plain", 	// the font name to use
				ff(18), 						// the point size to use
				&layoutOptions, 				// our default layout options
				&posn, 							// the shape's position
				0, 								// attributes (none)
				&runControls,					// our run controls
				toggleCaseFeatures, 			// the run features array (none)
				1, 								// count of one feature = 1
				nil);							// no style run overrides
				
	
// Retrieve the page shape so we can add to it.

	thePage = GetDocShape(wind);
	
// Add the layout to the window's picture shape.  AddToShape is in shape library.c.

	AddToShape(thePage, layout1);
	AddToShape(thePage, layout2);
	AddToShape(thePage, layout3);
	AddToShape(thePage, layout4);

// Now that all shapes are added to the picture, dispose of them

	GXDisposeShape(layout1);
	GXDisposeShape(layout2);
	GXDisposeShape(layout3);
	GXDisposeShape(layout4);
	

// Invalidate the window's portRect so that everything gets updated.
	
	SetPort(wind);
	InvalRect(&wind->portRect);
}

/*------ DoCreateNew -----------------------------------------------------------------------*/
//
//	This routine is called when a window needs to be created.
//
OSErr DoCreateNew(void)
{
	OSErr		err;
	WindowPtr	wind;
	
// Create a window, attach a default gxViewPort to it, create and iInitialize our
// private data for it, and add a sample image to its page shape.

	wind = NewWindow(nil, &gWindowQDRect, gWindowTitle, true, noGrowDocProc,
	                 (WindowPtr)-1L, true, 0L);
	
	if (!wind)
		err = MemError();
	else
	{
		SetDefaultViewPort(GXNewWindowViewPort(wind));			
		err = DoInitialization(wind);
	}
	
	CreateSampleLayoutImage(wind);
	return err;
}


/*------ DoDispose -------------------------------------------------------------------------*/
//
//	This routine is called when a window needs to be disposed of.
//
void DoDispose(wind)
WindowPtr wind;
{
	TH_Doc	doc;
	
// You should always dispose of your GX graphics objects before tossing your window.
// Why?  It's generally good form and this approach guarantees that everything is
// disposed.  If you had not disposed of everything, the call to DisposeWindow should
// dispose of the objects. If you are running the debugging version of QuickDraw GX
// with notices set, you will receive a notice that you had not disposed of everything.
// You can turn notices on in this file by setting gDebugging = TRUE (above).
	
	doc = (TH_Doc)  GetWRefCon(wind);	// Remember, this is where we stored our private data.
	
	GXDisposeShape(GetDocShape(wind)); 	// Dispose of this doc's shape.
	GXDisposeJob(GetDocJob(wind));		// Dispose of this doc's print job.
	DisposHandle((Handle) doc);			// Dispose of our private data.

   	DisposeWindow(wind);				// Dispose of the window.
}


/*------ DoIdle ----------------------------------------------------------------------------*/
//
//	This routine is called to do things while we're idling through the event loop.
//
void DoIdle(WindowPtr wind)
{
#pragma unused (wind)
}

/*------ DoTearDown ------------------------------------------------------------------------*/
//
//	This routine is called just before we quit to remove anything persistent that might have
//  been setup by DoSetup().
//
void DoTeardown(void)
{
}

