/*
	Basic Layout.c
	
	This file demonstrates how to create a layout shape containing the following 
	text: Catch The Nasty Wave, Dude. Once you have created the layout shape, we active a
	few layout features to recieve the "as" ligature in "Nasty", swashes for the "C", "N", 
	and "D", the final form of "e" in "Dude", and the automatic kerning of "WAVE".
	
	For details about enabling the various layout features, take a look at the comments contained
	within the DoInitialization function.
		
	Enjoy,
	
	...Luke
	
	>> Pete "Luke" Alexander
	>> Apple Computer, Developer Technical Support
	
	HISTORY:
	
	4/93	New   PLA
	12/93	Updated the layout feature constants to reflect the changes to 3. See the comments
			within the DoInitialization function for complete details.  PLA
			
			
	NOTES:
	
	This code requires the following files: 
	  	"graphics shell.c", "color library.c", "font library.c", "graphics debug library.c",
	  	"layout library.c", "transform library.c"
	  	
	For the best printing results, print in "landscape".
	
	1990-1993 Apple Computer, Inc.
	All rights reserved.
*/


#include <Events.h>
#include <Windows.h>

#include "graphics types.h"
#include "graphics libraries.h"
#include "font library.h"
#include "graphics toolbox.h"
#include "graphics shell.h"

#include "layout library.h"
#include "layout routines.h"
#include "layout feature constants.h"


//
//  Set up the title and size of the window 
//
Str255 				gWindowTitle = "\p Basic Layout ";
Rect 				gWindowQDRect  = {40, 10, 155, 580};

//
//	If gDebugging = TRUE, graphics library errors and notices will be posted.  This functionality  will only work with 
//	the "debugging" version of the Secret Graphics init.  If this version of the init is not installed, nothing bad will happen, 
//	but these  functions will not work. The "debugging" version of the Secret Graphics init is approximately 1024K.
//
Boolean				gDebugging = true;

//
//	 Set  "gGiveMeValidation" to TRUE, if you will receive run-time validation.  
//
Boolean				gGiveMeValidation = true;

//
//	gGraphicsHeapSize sets the size of the graphics heap created by calling the NewGraphicsClient routine
//	in main () within graphics shell.c.  You can determine the amount of graphics heap required by using GraphicsBug.
//	With  gGraphicsHeapSize set to 25k, I had 2 free blocks left in the graphics heap. Sounds good to me.
//
long				gGraphicsHeapSize = 125;

gxShape 			gThePage;

short				gClickMe;
//gxStyle				styles[3];
//gxShape 			tempLayoutShape;
//short				totalLengthOfLayout, 
//					lengthsArray[3],
//					loop;

//gxRunControls		runControls;
//gxRunFeature		runFeatures[3];
//gxPoint 			layoutPostion = {ff(10), ff(65)};


/*------ DoInitialization ---------------------------------------------------------------------------------*/

void DoInitialization(aWindow)
WindowPtr aWindow;
{
	char 				*sampleText = "Catch the Nasty WAVE, Dude";
	gxPoint 			layoutPostion = {ff(10), ff(65)};
	gxRunControls		runControls;
	gxRunFeature		runFeatures[3];
	gxShape 			tempLayoutShape;
	gxStyle				styles[3];
	short				totalLengthOfLayout, 
						lengthsArray[3],
						loop;
	
	gClickMe = 1;

	//
	//	NOTE: 	I'm collecting everything into a picture for easy printing. The picture would
	//			be passed to the GX printing system with both shapes: text and layout
	//				
	gThePage = GXNewShape (gxPictureType);
	GXSetShapeAttributes (gThePage, gxUniqueItemsShape);


	//
	//	At this point, we have converted our "tempLayoutShape" into a layout sahpe. This shape
	//	is now ready to have the various line layout features turned on.
	//	
	//	We start by intializing the layout run controls to their default values.
	//
	InitializeRunControls(&runControls);
		

	//
	//	We need to create the style which will be used for the first run of text within our
	//	layout shape: "Catch the Nasty". Within a layout shape, you have the ability to have
	//	multiple runs of text. Each run can use: a different font, text size, run features,
	//	run controls.
	//	
	//	In the case of our first run of text, we want to use Hoefler Italic and a text size of 45.
	//	We set the run controls to their default setting. 
	//
	styles[0] = NewLayoutStyle((char *)"\pHoefler Italic", ff(42), 0, &runControls, nil, 0, nil);


	//
	//	Next, we want to turn on a couple of run features within this run of text. We start by
	//	enabling the "as" ligature in "Nasty". We tell line layout that we want to set the 
	//	featureType of our run control to enable the ligature capabilities of Hoefler Italic, and
	//	we then enable the "as" ligature. 
	//
	runFeatures[0].featureType = ligaturesType;
	runFeatures[0].featureSelector = ligatureRareOnSelector;


	//
	//	!!  3 Change  !!
	//
	//	The next thing we need to tell line layout to do is enable the swash capabilities of our 
	//	"C" and "N" characters contained within the first run of text. These swashes are part of the 
	//	Hoefler Italic font. We acheive this goal by setting the runFeatures featureType
	//	to characterAlternativesType with a featureSelector of 1.
	//
	//	This is an example were the characterAlternativesType feature is font dependent. Each TrueType
	//	GX font can contain 0 or more additional character alternatives selectors defined by a font.
	//	In the case of the Hoefler Italic font, the 1st feature enables the "C" and "N" swashes of 
	//	each word.
	//
	runFeatures[1].featureType = characterAlternativesType;
	runFeatures[1].featureSelector = 1;
	
	//
	//	After we have created the run features we need, we set the run features of our
	//	style to use them.
	//
	GXSetStyleRunFeatures(styles[0], 2, runFeatures);
	
	//	 
	//	In the second style we need to create is for the "WAVE!" run. For this run of text, 
	//	we want the font to be Times Roman, 50 point text size, and use the default run 
	//	controls. This style will also give us automatic kerning of the run.
	//
	styles[1] = NewLayoutStyle((char *)"\pTimes Roman", ff(44), 0, &runControls, nil, 0, nil);
	

	//
	//	The final style we need to create is used by the last run of text "dude". 
	//
	styles[2] = NewLayoutStyle((char *)"\pHoefler Italic", ff(47), 0, &runControls, nil, 0, nil);

	//
	//	The run features used for the style reference by the last run of text will
	//	enable the final form of "e" in "dude". We enable these smartSwashType feature and
	//	turn on the final form of "e".
	//
	runFeatures[2].featureType = smartSwashType;
	runFeatures[2].featureSelector = lineFinalSwashesOnSelector;
	
	//
	//	Update the style used for this run of text to use the our run features which
	//	enable the final form "e".
	//
	GXSetStyleRunFeatures(styles[2], 3, runFeatures);

	//
	//	The "lengthsArray" defines the length of each run of text used within our layout shape. 
	//	In our case, the first run of text is 16 characters, the second run is 4 characters,
	//	and the final run is 5 characters.
	//
	lengthsArray[0] = 15;    lengthsArray[1] = 6;  lengthsArray[2] = 5;

	totalLengthOfLayout = 26;

	//
	//	We now update our layout shape to include our 2 new styles. When our layout shape
	//	is drawn, the "C" and "N" swashes, "as" ligature, and the kerning of "WAVE" will
	//	appear.
	//
	tempLayoutShape = GXNewLayout(1, &totalLengthOfLayout, (void *) &sampleText, 
							      3, lengthsArray, styles, 0, nil, nil, nil, &layoutPostion);

	GXSetPictureParts(gThePage, 0, 0, 1, &tempLayoutShape, nil, nil, nil);

	//
	//	The tempLayoutShape is now referenced by the picture, therefore we can dispose of it.
	//
	GXDisposeShape (tempLayoutShape);

  	//
  	//	We can now dispose of our styles because they have been added into our layout shape.
  	//	They are no longer needed....
  	//
	for (loop = 0; loop < 3; loop++) 
	  GXDisposeStyle(styles[loop]);
}


/*------ DoClick ---------------------------------------------------------------------------------------*/

void DoClick(aWindow)
WindowPtr aWindow;
{

	switch (gClickMe)
	{
		case 1:
		{
		break;
		}
		
		case 2:
		{
			break;
		}
		
		case 3:
		{
		}
	
		default: break;
	
	}

	gClickMe++;
	
	DoDraw(aWindow);

}

/*------ DoDraw ---------------------------------------------------------------------------------------*/

void DoDraw(aWindow)
WindowPtr aWindow;
{
	GXDrawShape (gThePage);
}


/*------ DoDispose -------------------------------------------------------------------------------------*/

void DoDispose(aWindow)
WindowPtr aWindow;
{
	/**  
		You should always dispose of your GX graphics objects before tossing your window. Why? It's generally good 
		form and this approach guarantees that everything is disposed. If you had not disposed of everything, the
		call to DisposeWindow should dispose of the objects. If you are running the debugging version of the 
		SecretGraphics init with notices set, you will receive a notice that you had not disposed of everything. You
		can turn notices on in this file by setting gDebugging = TRUE (above).
	**/
	GXDisposeShape(gThePage);  
	GXDisposeShape(gWindowBoundsShape);  
   	DisposeWindow(aWindow);
}
	



/*------ DoIdle ----------------------------------------------------------------------------------------*/

void DoIdle(aWindow)
WindowPtr aWindow;
{
}
